<?php if ( ! defined( 'ABSPATH' ) ) exit;

class DDBUILDER_MediaLibrary {
    protected $category_slug  = 'dondivi_media_category';

    public function __construct() {
        add_action( 'init', array( $this, 'categories_init' ) );
        
        if ( is_admin() ) {
            add_filter( 'attachment_fields_to_edit', array( $this, 'attachment_fields_to_edit' ), 10, 2 );
            add_filter( 'attachment_fields_to_save', array( $this, 'attachment_fields_to_save' ), 10, 2) ;
        } 
    }
    
    public function categories_init() {
       
        $taxonomy = $this->category_slug;

        if ( taxonomy_exists( $taxonomy ) ) {
            register_taxonomy_for_object_type( $taxonomy, 'attachment' );
        } else {
            $labels = array(
                'name'              => _x( 'DonDivi Media Categories', 'taxonomy general name', 'dd_builder' ),
                'singular_name'     => _x( 'DonDivi Media Category', 'taxonomy singular name', 'dd_builder' ),
                'search_items'      => __( 'Search Categories', 'dd_builder' ),
                'all_items'         => __( 'All Categories', 'dd_builder' ),
                'parent_item'       => __( 'Parent Category', 'dd_builder' ),
                'parent_item_colon' => __( 'Parent Category:', 'dd_builder' ),
                'edit_item'         => __( 'Edit Category', 'dd_builder' ),
                'update_item'       => __( 'Update Category', 'dd_builder' ),
                'add_new_item'      => __( 'Add New Category', 'dd_builder' ),
                'new_item_name'     => __( 'New Category Name', 'dd_builder' ),
                'menu_name'         => __( 'DonDivi Categories', 'dd_builder' ),
            );
            $args = array(
                'hierarchical'          => true,
                'labels'                => $labels,
                'show_admin_column'     => true,
                'show_in_rest'          => true,
            );
            register_taxonomy( $taxonomy, array( 'attachment' ), $args );
        }
    }

    function attachment_fields_to_edit( $form_fields, $post ) { 
        
        if (isset($form_fields[$this->category_slug])) $form_fields[$this->category_slug]['helps'] = esc_html__('This will allow you to filter your images in the DonDivi Gallery', 'dd_builder');
        
        $form_fields['ddbuilder_url_field'] = array(
            'label' => esc_html__('DonDivi Destination URL', 'dd_builder'),
            'input' => 'text',
            'value' => esc_url( get_post_meta($post->ID, '_ddbuilder_url', true) ),
            'helps' => esc_html__('This will allow you to open this URL in the DonDivi Gallery', 'dd_builder') 
        );

        $featured = (bool) get_post_meta($post->ID, '_ddbuilder_featured', true);
        $form_fields['ddbuilder_featured_field'] = array(
            'label' => esc_html__('DonDivi Featured Image', 'dd_builder'),
            'input' => 'html',
            'html'  => sprintf('<input type="checkbox" %1$s name="attachments[%2$s][ddbuilder_featured_field]" id="attachments-%2$s-ddbuilder_featured_field" />', $featured ? ' checked' : '', $post->ID),  
            'value' => $featured,
            'helps' => esc_html__('This will allow you to feature this image in the DonDivi Gallery', 'dd_builder') 
        );
        return $form_fields;
    }
    
    function attachment_fields_to_save($post, $attachment) {  
        if( isset($attachment['ddbuilder_url_field']) ){  
            update_post_meta($post['ID'], '_ddbuilder_url', esc_url_raw( $attachment['ddbuilder_url_field'] ) );
        }
        update_post_meta($post['ID'], '_ddbuilder_featured', isset( $attachment['ddbuilder_featured_field'] ) ? '1' : '0');  
        return $post;  
    }
  
}

new DDBUILDER_MediaLibrary();