<?php

class DDBUILDER_DonDiviBuilder extends DiviExtension {

	/**
	 * The gettext domain for the extension's translations.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	public $gettext_domain = 'dd_builder';

	/**
	 * The extension's WP Plugin name.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	public $name = 'dondivi-builder';

	/**
	 * The extension's version
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	public $version = DDB_VERSION;

	/**
	 * DDBUILDER_DonDiviBuilder constructor.
	 *
	 * @param string $name
	 * @param array  $args
	 */
	public function __construct( $name = 'dondivi-builder', $args = array() ) {
		$this->plugin_dir     = plugin_dir_path( __FILE__ );
		$this->plugin_dir_url = plugin_dir_url( $this->plugin_dir );

		parent::__construct( $name, $args );

		$defaults = array(
			'title'	=> _x( 'Your Title Goes Here', 'Modules dummy content', 'et_builder' ),
			'body'	=> _x( 'Your content goes here. Edit or remove this text inline or in the module Content settings. You can also style every aspect of this content in the module Design settings and even apply custom CSS to this text in the module Advanced settings.',
			'et_builder' ),
			'image' => ET_BUILDER_PLACEHOLDER_LANDSCAPE_IMAGE_DATA
		);

		$this->_builder_js_data	= array(
			'all' 		=> esc_html__( 'All', 'et_builder' ),
			'card'		=> array(
				'cc_title' 	=> $defaults['title'],
				'cc_image' 	=> $defaults['image'],  
				'content'	=> $defaults['body'],
			),
			'hotspots'		=> array(
				'src' 	=> $defaults['image'],
			),
			'hotspot'		=> array(
				'cc_image' 	=> $defaults['image'],  
				'content'	=> $defaults['body'],
			),
			'tab'  		=> array(
				'title'   	=> _x( 'Tab Title', 'Modules dummy content', 'et_builder' ),
				'cc_title' 	=> $defaults['title'],
				'cc_image' 	=> $defaults['image'],
				'content'  	=> $defaults['body']
			),
			'toggle' 	=> array(
				'label1' 		=> $defaults['title'],
				'label2' 		=> $defaults['title'],
				'c1_content'   	=> $defaults['body'],
				'c1_cc_image' 	=> $defaults['image'],
				'c2_content'   	=> $defaults['body'],
				'c2_cc_image' 	=> $defaults['image'],
			),
			'image_data' 	=> $defaults['image'],
			'close_cursor' 	=> DDB_CLOSE_CURSOR,
		);

		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
		add_filter( 'et_builder_module_dondivi_menu_item_outer_wrapper_attrs', array($this, 'menu_item_outer_wrapper_attrs'), 10, 2);
		add_filter( 'et_pb_all_fields_unprocessed_dondivi_content_toggle', array($this, 'content_toggle_finalize_fields') );
		add_filter( 'et_pb_all_fields_unprocessed_dondivi_gallery', array($this, 'gallery_finalize_fields') );
		add_filter( 'et_pb_all_fields_unprocessed_dondivi_grid_item', array($this, 'item_content_finalize_fields') );
		add_filter( 'et_pb_all_fields_unprocessed_dondivi_hotspot', array($this, 'hotspot_finalize_fields') );
		add_filter( 'et_pb_all_fields_unprocessed_dondivi_menu_item', array($this, 'menu_item_finalize_fields') );
		add_filter( 'et_pb_all_fields_unprocessed_dondivi_popup', array($this, 'item_content_finalize_fields') );
		add_filter( 'et_pb_all_fields_unprocessed_dondivi_tab', array($this, 'item_content_finalize_fields') );
		add_filter( 'et_pb_all_fields_unprocessed_dondivi_timeline_item', array($this, 'timeline_finalize_fields') );
		add_filter( 'et_required_module_assets', array($this, 'button_assets') );
		add_filter( 'woocommerce_add_to_cart_fragments', array( $this, 'update_cart' ) );		 
	}
	
	public function enqueue_scripts() {
		wp_register_script( 'magnific-popup', $this->plugin_dir_url . 'builder/js/jquery.magnific-popup.min.js', array('jquery'), false, true );
		wp_register_script( 'ddb-imagesloaded', $this->plugin_dir_url . 'builder/js/imagesloaded.pkgd.min.js', array('jquery'), false, true );
		wp_register_script( 'ddb-masonry', $this->plugin_dir_url . 'builder/js/masonry.pkgd.min.js', array('jquery'), false, true );
		
		wp_register_script( 'ddb-gallery', $this->plugin_dir_url . 'includes/modules/Gallery/script.min.js', array( 'jquery', 'magnific-popup' ), false, true );
		wp_register_script( 'ddb-grid', $this->plugin_dir_url . 'includes/modules/Grid/script.min.js', array('jquery'), false, true );
		wp_register_script( 'ddb-hotspots', $this->plugin_dir_url . 'includes/modules/Hotspots/script.min.js', array('jquery'), false, true );
		wp_register_script( 'ddb-menu', $this->plugin_dir_url . 'includes/modules/NavMenu/script.min.js', array('jquery'), false, true );
		wp_register_script( 'ddb-popup', $this->plugin_dir_url . 'includes/modules/Popup/script.min.js', array('jquery'), false, true );
		wp_register_script( 'ddb-tabs', $this->plugin_dir_url . 'includes/modules/Tabs/script.min.js', array('jquery'), false, true );
		wp_register_script( 'ddb-timeline', $this->plugin_dir_url . 'includes/modules/Timeline/script.min.js', array('jquery'), false, true );
		wp_register_script( 'ddb-toggle', $this->plugin_dir_url . 'includes/modules/ContentToggle/script.min.js', array('jquery'), false, true );

		
		if (function_exists('et_core_is_fb_enabled') && et_core_is_fb_enabled()) {
			wp_enqueue_script( 'magnific-popup' );
			wp_enqueue_script( 'ddb-imagesloaded' );
			wp_enqueue_script( 'ddb-masonry' );
		}
    }

	public function button_assets( $modules ) {
        array_push( $modules, 'et_pb_button' );
        return $modules;
    }

	public function content_toggle_finalize_fields($fields) {
		$c1_custom_content = array( 'item_content1' => 'custom');
		$this->set_font_fields_visibility( $fields, [ 'c1_title' ], $c1_custom_content ); 
		$this->set_font_fields_visibility( $fields, [ 'c1_body', 'c1_body_link', 'c1_body_ul', 'c1_body_ol', 'c1_body_quote' ], $c1_custom_content );
		$fields['image_icon_custom_margin']['show_if'] = array( 'item_content1' => 'custom', 'c1_cc_icon_image' => array( 'image', 'icon' ) );
		$fields['image_icon_custom_padding']['show_if'] = array( 'item_content1' => 'custom', 'c1_cc_icon_image' => array( 'image', 'icon' ) );
		$fields['custom_c1_button']['show_if'] = $fields['c1_button_alignment']['show_if'] = $c1_custom_content;
		$c2_custom_content = array( 'item_content2' => 'custom');
		$this->set_font_fields_visibility( $fields, [ 'c2_title' ], $c2_custom_content ); 
		$this->set_font_fields_visibility( $fields, [ 'c2_body', 'c2_body_link', 'c2_body_ul', 'c2_body_ol', 'c2_body_quote' ], $c2_custom_content );
		$fields['image_icon_2_custom_margin']['show_if'] = array( 'item_content2' => 'custom', 'c2_cc_icon_image' => array( 'image', 'icon' ) );
		$fields['image_icon_2_custom_padding']['show_if'] = array( 'item_content2' => 'custom', 'c2_cc_icon_image' => array( 'image', 'icon' ) );
		$fields['custom_c2_button']['show_if'] = $fields['c2_button_alignment']['show_if'] = $c2_custom_content;
		return $fields;
	}

	public function gallery_finalize_fields($fields) {
		$fields['filter_custom_margin']['sub_toggle'] = 'filters';
		$fields['filter_custom_padding']['sub_toggle'] = 'filters';
		$fields['page_custom_margin']['sub_toggle'] = 'pagination';
		$fields['page_custom_padding']['sub_toggle'] = 'pagination';
		$fields['overlay_custom_padding']['show_if'] = array( 'overlay' => 'on');
		return $fields;
	}  

	public function hotspot_finalize_fields( $fields ) {
		$custom_content = array( 'link_type' => array( 'tooltip', 'popup_c' ) );
		$this->set_font_fields_visibility( $fields, [ 'title' ], $custom_content );
		$this->set_font_fields_visibility( $fields, [ 'body', 'body_link', 'body_ul', 'body_ol', 'body_quote' ], $custom_content );
		$fields['image_icon_custom_margin']['show_if'] = array( 'link_type' => array( 'tooltip', 'popup_c' ), 'cc_icon_image' => array( 'image', 'icon' ) );
		$fields['image_icon_custom_padding']['show_if'] = array( 'link_type' => array( 'tooltip', 'popup_c' ), 'cc_icon_image' => array( 'image', 'icon' ) );
		$fields['tooltip_custom_padding']['show_if'] = array( 'link_type' => 'tooltip' );
		$fields['popup_custom_padding']['show_if'] = array( 'link_type' => array( 'popup_c', 'popup_l') );
		$fields['custom_button']['show_if'] = $fields['button_alignment']['show_if'] = $custom_content;
		return $fields;	
	}

	public function menu_finalize_fields( $fields ) {
		$fields['mobile_icon_custom_margin']['show_if'] = array('enable_mobile' => 'on');
		$fields['mobile_icon_custom_padding']['show_if'] = array('enable_mobile' => 'on');
		return $fields;
	}

	public function menu_item_finalize_fields( $fields ) {
		$custom_content = array( 'link_type' => array( 'sub_c', 'popup_c') );
		$this->set_font_fields_visibility( $fields, [ 'title' ], $custom_content );
		$this->set_font_fields_visibility( $fields, [ 'body', 'body_link', 'body_ul', 'body_ol', 'body_quote' ], $custom_content );
		$fields['image_icon_custom_margin']['show_if'] = array( 'link_type' => array( 'sub_c', 'popup_c' ), 'cc_icon_image' => array( 'image', 'icon' ) );
		$fields['image_icon_custom_padding']['show_if'] = array( 'link_type' => array( 'sub_c', 'popup_c' ), 'cc_icon_image' => array( 'image', 'icon' ) );
		$fields['toggle_icon_custom_padding']['show_if'] = array( 'link_type' => array( 'sub_c', 'sub_l', 'sub_wp' ), 'use_toggle_icon' => 'on' );
		$fields['popup_custom_padding']['show_if'] = array( 'link_type' => array( 'popup_c', 'popup_l' ) );
		$fields['sub_custom_padding']['show_if'] = array( 'link_type' => array( 'sub_c', 'sub_l', 'sub_wp' ) );
		$fields['wp_custom_padding']['show_if'] =  array('link_type' => 'sub_wp');
		$fields['wp_inner_custom_padding']['show_if'] =  array('link_type' => 'sub_wp');
		$fields['custom_button']['show_if'] = $fields['button_alignment']['show_if'] = array( 'link_type' => array( 'sub_c', 'popup_c' ) );
		return $fields;
	}

	public function item_content_finalize_fields($fields) {
		$custom_content = array( 'item_content' => 'custom' );
		$this->set_font_fields_visibility( $fields, [ 'title' ], $custom_content );
		$this->set_font_fields_visibility( $fields, [ 'body', 'body_link', 'body_ul', 'body_ol', 'body_quote' ], $custom_content );
		$fields['image_icon_custom_margin']['show_if'] = array( 'item_content' => 'custom', 'cc_icon_image' => array( 'image', 'icon' ) );
		$fields['image_icon_custom_padding']['show_if'] = array( 'item_content' => 'custom', 'cc_icon_image' => array( 'image', 'icon' ) );
		$fields['custom_button']['show_if'] = $fields['button_alignment']['show_if'] = $custom_content;
		if ( isset( $fields['custom_trigger'] ) ) $fields['custom_trigger']['show_if'] = array( 'popup_type' => 'button' );
		return $fields;
	}
	
	public function menu_item_outer_wrapper_attrs($attrs, $instance) {
		$attrs['role']= 'menuitem';
		return $attrs;
	}

	public function timeline_finalize_fields( $fields ) {
		$fields['image_icon_custom_margin']['show_if'] = array( 'cc_icon_image' => array( 'image', 'icon' ) );
		$fields['image_icon_custom_padding']['show_if'] = array( 'cc_icon_image' => array( 'image', 'icon' ) );
		return $fields;
	} 

	public function update_cart( $fragments ) {
    	$fragments['.ddb-cart-count'] = sprintf('<span class="ddb-cart-info ddb-light ddb-f ddb-f-cc ddb-abs ddb-tr ddb-cart-count">%s</span>', WC()->cart->get_cart_contents_count());
    	return $fragments; 
	}

	private function set_font_fields_visibility( &$fields, $elements, $show_if_array ) {	
		foreach ( $elements as $element) {
			if ( isset($fields[$element .'_level'] ) ) $fields[$element .'_level']['show_if'] = $show_if_array;
			if ( isset($fields[$element .'_font'] ) ) $fields[$element .'_font']['show_if'] = $show_if_array;
			if ( isset($fields[$element .'_font_size'] ) ) $fields[$element .'_font_size']['show_if'] = $show_if_array;
			if ( isset($fields[$element .'_letter_spacing'] ) ) $fields[$element .'_letter_spacing']['show_if'] = $show_if_array;
			if ( isset($fields[$element .'_line_height'] ) ) $fields[$element .'_line_height']['show_if'] = $show_if_array;
			if ( isset($fields[$element .'_text_align'] ) ) $fields[$element .'_text_align']['show_if'] = $show_if_array;
			if ( isset($fields[$element .'_text_color'] ) ) $fields[$element .'_text_color']['show_if'] = $show_if_array;
			if ( isset($fields[$element .'_type'] ) ) $fields[$element .'_type']['show_if'] = $show_if_array;
			if ( isset($fields[$element .'_position'] ) ) $fields[$element .'_position']['show_if'] = $show_if_array;
			if ( isset($fields[$element .'_item_indent'] ) ) $fields[$element .'_item_indent']['show_if'] = $show_if_array;
			if ( isset($fields[$element .'_border_weight'] ) ) $fields[$element .'_border_weight']['show_if'] = $show_if_array;
			if ( isset($fields[$element .'_border_color'] ) ) $fields[$element .'_border_color']['show_if'] = $show_if_array;
		}
	}
	
}

new DDBUILDER_DonDiviBuilder;
