<?php if ( ! defined( 'ABSPATH' ) ) exit;


class DonDiviBuilderFields {

	private static $layouts = array();
	private static $all_layouts = array();

	public function get_all_layouts() {
		if (empty(self::$all_layouts)) {
			$posts = get_posts(array('post_type' => 'et_pb_layout', 'numberposts' => -1)); 
			self::$all_layouts[] = __('-- Select --', 'dd_builder');
			foreach ($posts as $post) {	
				self::$all_layouts[$post->ID] = $post->post_title;		
			}
		}
		return self::$all_layouts;
	}

	public function get_alt_field( $show_if, $label ) {
		$alt = array( 
			'label' 		=> esc_html__( $label, 'et_builder' ),
			'description'  	=> esc_html__( 'This defines the HTML ALT text. A short description of your image can be placed here.', 'et_builder' ),
			'type'         	=> 'text',
			'tab_slug'     	=> 'custom_css',
			'toggle_slug'  	=> 'attributes'
		);
		if ($show_if !== '') $alt['show_if'] = $show_if;
		return $alt;
	}

	public function get_custom_class_field() {
		return array(
			'label'           => esc_html__( 'CSS Class', 'et_builder' ),
			'description'     => esc_html__( "Assign any number of CSS Classes to the element, separated by spaces, which can be used to assign custom CSS styles from within your child theme or from within Divi's custom CSS inputs.", 'et_builder' ),
			'type'            => 'text',
			'tab_slug'        => 'custom_css',
			'toggle_slug'     => 'classes',
			'option_class'    => 'et_pb_custom_css_regular',
		);
	}

	public function get_custom_content_fields($settings) {
		$prefix = isset($settings['prefix']) ? $settings['prefix'] . '_' : ''; 
		$fields = array(
			"{$prefix}cc_title"	=> array(
				'label'           	=> et_builder_i18n( 'Title' ),
				'type'            	=> 'text',
				'dynamic_content' 	=> 'text',
				'mobile_options'  	=> true,
				'hover'           	=> 'tabs',
				'toggle_slug'     	=> $settings['content_slug'],
			),
			"{$prefix}cc_url"  	=> array(
				'label'           => esc_html__( 'Title Link URL', 'et_builder' ),
				'type'            => 'text',
				'toggle_slug'     => isset($settings['link_slug']) ? $settings['link_slug'] : $settings['advanced_slug'],
				'dynamic_content' => 'url',
			),
			"{$prefix}cc_url_new_window"  => array(
				'label'            => esc_html__( 'Title Link Target', 'et_builder' ),
				'description'      => esc_html__( 'Here you can choose whether or not your link opens in a new window', 'et_builder' ),
				'type'             => 'select',
				'options'          => array(
					'off' => esc_html__( 'In The Same Window', 'et_builder' ),
					'on'  => esc_html__( 'In The New Tab', 'et_builder' ),
				),
				'default' => 'off',
				'toggle_slug'      => isset($settings['link_slug']) ? $settings['link_slug'] : $settings['advanced_slug'],			
			),
			"{$prefix}content" 		=> array(
				'label'           	=> et_builder_i18n( 'Body' ),
				'description'     	=> esc_html__( 'Input the main text content for your module here.', 'et_builder' ),
				'type'            	=> 'tiny_mce',
				'dynamic_content' 	=> 'text',
				'mobile_options'  	=> true,
				'hover'           	=> 'tabs',
				'toggle_slug'     	=> $settings['content_slug'],
			),
		);

		$icon_fields = isset($settings['icon']) && $settings['icon'] ? array(
			"{$prefix}cc_icon_image"    =>  array(
				'label'           	=> et_builder_i18n( 'Image/Icon' ),
				'type'            	=> 'multiple_buttons',
				'options'         	=>  array(
					'none' 	=>  array( 'title' 	=>  esc_html__('None', 'et_builder') ),
					'icon' 	=>  array( 'title' 	=>  esc_html__('Icon', 'et_builder') ),
					'image' =>  array( 'title'  =>  esc_html__('Image', 'et_builder') ),
				),
				'default'         	=> 'none',
				'toggle_slug'     	=> $settings['content_slug'],
			),
			"{$prefix}cc_icon" 		=> array(
				'label'           	=> esc_html__( 'Icon', 'et_builder' ),
				'type'            	=> 'select_icon',
				'class'           	=> array( 'et-pb-font-icon' ),
				'show_if'		  	=> array( "{$prefix}cc_icon_image" => 'icon' ),
				'mobile_options'  	=> true,
				'hover'				=> 'tabs',
				'toggle_slug'       => $settings['content_slug'],		
			),
			"{$prefix}cc_icon_color" => array(
				'label'           	=> esc_html__( 'Icon Color', 'et_builder' ),
				'type'            	=> 'color-alpha',
				'default'         	=> et_builder_accent_color(),
				'hover'           	=> 'tabs',
				'mobile_options'  	=> true,
				'sticky'          	=> true,
				'show_if' 			=> array( "{$prefix}cc_icon_image" => 'icon'),
				'tab_slug'        	=> 'advanced',
				'toggle_slug'     	=> isset($settings['icon_slug']) ? $settings['icon_slug'] : $settings['advanced_slug'],
			),
			"{$prefix}cc_image" 		=> array(
				'label'             => esc_html__( 'Image', 'et_builder' ),
				'type'              => 'upload',
				'upload_button_text'=> esc_attr__( 'Upload an image', 'et_builder' ),
				'choose_text'       => esc_attr__( 'Choose an Image', 'et_builder' ),
				'update_text'       => esc_attr__( 'Set As Image', 'et_builder' ),
				'show_if'			=> array( "{$prefix}cc_icon_image" => 'image'),
				'mobile_options'  	=> true,
				'hover'				=> 'tabs',
				'dynamic_content'   => 'image',
				'toggle_slug'       => $settings['content_slug'],
			),
			"{$prefix}cc_icon_width" 	=> array(
				'label'                	=> esc_html__( 'Image/Icon Width', 'et_builder' ),	
				'type'                 	=> 'range',
				'default'				=> '50px',
				'default_unit'			=> 'px',
				'show_if'				=> array( "{$prefix}cc_icon_image" => 'icon' ),
				'mobile_options'        => true,
				'tab_slug'              => 'advanced',
				'toggle_slug'         	=> isset($settings['icon_slug']) ? $settings['icon_slug'] : $settings['advanced_slug'],
			),
			"{$prefix}cc_image_width" 	=> array(
				'label'                	=> esc_html__( 'Image/Icon Width', 'et_builder' ),	
				'type'                 	=> 'range',
				'default'				=> 'auto',
				'default_unit'			=> '%',
				'allowed_values' 		=> et_builder_get_acceptable_css_string_values( 'width' ),
				'show_if'				=> array( "{$prefix}cc_icon_image" => 'image' ),
				'mobile_options'        => true,
				'tab_slug'              => 'advanced',
				'toggle_slug'         	=> isset($settings['icon_slug']) ? $settings['icon_slug'] : $settings['advanced_slug'],
			),
			"{$prefix}cc_alt" => array(
				'label'           	=> esc_html__( 'Custom Image Alt Text', 'dd_builder' ),
				'description'     	=> esc_html__( 'Define the HTML ALT text for your image here.', 'et_builder' ),
				'type'            	=> 'text',
				'show_if'			=> array( "{$prefix}cc_icon_image" => 'image' ),
				'tab_slug'        	=> 'custom_css',
				'toggle_slug'     	=> 'attributes',
				'dynamic_content' 	=> 'text',
			),
			"{$prefix}cc_icon_placement"              => array(
				'label'            => esc_html__( 'Image/Icon Placement', 'et_builder' ),
				'description'      => esc_html__( 'Here you can choose where to place the icon.', 'et_builder' ),
				'type'             => 'select',
				'options'          => array(
					'column' => et_builder_i18n( 'Top' ),
					'row' => et_builder_i18n( 'Left' ),
				),
				'default' 		=> 'column',
				'show_if'		   => array( "{$prefix}cc_icon_image" => array('icon', 'image')),				
				'mobile_options'   => true,
				'tab_slug'         => 'advanced',
				'toggle_slug'      => isset($settings['icon_slug']) ? $settings['icon_slug'] : $settings['advanced_slug'],
			),
			"{$prefix}cc_icon_alignment"              => array(
				'label'           => esc_html__( 'Image/Icon Alignment', 'et_builder' ),
				'description'     => esc_html__( 'Align image/icon to the left, right or center.', 'et_builder' ),
				'type'            => 'align',
				'options'         => et_builder_get_text_orientation_options( array( 'justified' ) ),
				'tab_slug'        => 'advanced',
				'toggle_slug'     => isset($settings['icon_slug']) ? $settings['icon_slug'] : $settings['advanced_slug'],
				'default'         => 'center',
				'mobile_options'  => true,
				'show_if'         => array(
					"{$prefix}cc_icon_image" => array('icon', 'image'),
					"{$prefix}cc_icon_placement" => 'column',
				),
			),
			"{$prefix}cc_icon_vertical_alignment" => array( 
				'label'           	=> esc_html__( 'Image/Icon Alignment', 'et_builder' ),
				'type'            	=> 'select',
				'options'         	=> array('flex-start' => esc_html__( 'Top', 'et_builder' ), 'center' => esc_html__( 'Middle', 'et_builder' ), 'flex-end' => esc_html__( 'Bottom', 'et_builder' )),
				'default'			=> 'flex-start',
				'mobile_options'	=> true,
				'tab_slug'        	=> 'advanced',
				'toggle_slug'            => isset($settings['icon_slug']) ? $settings['icon_slug'] : $settings['advanced_slug'],
				'show_if'         => array(
					"{$prefix}cc_icon_image" => array('icon', 'image'),
					"{$prefix}cc_icon_placement" => 'row',
				),
			),
			"{$prefix}cc_gap" => array(
				'label'           	=> sprintf('%1$s %2$s',  et_builder_i18n( 'Image/Icon' ), esc_html__( 'Offset', 'dd_builder' ) ),
				'description'     	=> esc_html__( 'Set the distance between the Image/Icon and the content Title/Body', 'dd_builder' ),
				'type'            	=> 'range',
				'default'			=> '20px', 
				'default_unit'		=> 'px',
				'show_if'		  	=> array( "{$prefix}cc_icon_image" => array('icon', 'image')),
				'mobile_options'  	=> true,
				'sticky'			=> true,
				'tab_slug'       	=> 'advanced',
				'toggle_slug'      	=> isset($settings['icon_slug']) ? $settings['icon_slug'] : $settings['advanced_slug'],			
			),
		) : array();
		
		
		$button_fields = isset($settings['button']) && $settings['button'] ? array(
			"{$prefix}cc_button_text"    => array(
				'label'           	=> et_builder_i18n( 'Button' ),
				'description'     	=> esc_html__( 'Input your desired button text, or leave blank for no button.', 'et_builder' ),
				'type'            	=> 'text',
				'mobile_options'  	=> true,
				'hover'           	=> 'tabs',
				'dynamic_content' 	=> 'text',
				'toggle_slug'     	=> $settings['content_slug'],
			),
			"{$prefix}cc_button_url"     => array(
				'label'          	=> esc_html__( 'Button Link URL', 'et_builder' ),
				'type'            	=> 'text',
				'toggle_slug'     	=> $settings['content_slug'],
				'dynamic_content'	=> 'url',
			),
			"{$prefix}cc_button_new_window" => array(
				'label'            	=> esc_html__( 'Button Link Target', 'et_builder' ),
				'description'      	=> esc_html__( 'Here you can choose whether or not your link opens in a new window', 'et_builder' ),
				'type'             	=> 'select',
				'options'          	=> array(
					'off' => esc_html__( 'In The Same Window', 'et_builder' ),
					'on'  => esc_html__( 'In The New Tab', 'et_builder' ),
				),
				'default' 			=> 'off',
				'toggle_slug'      	=> $settings['content_slug'],
			),
		) : array();
		
		$fields = array_merge( $icon_fields, $fields, $button_fields ); 
		foreach ($fields as $field => $field_settings) {
			if (isset($settings['show_if_not']))
				$fields[$field]['show_if_not'] = $settings['show_if_not'];
		}
		return $fields; 
	}

	public function get_effect_options() {
		return array(
			'none' 			=> esc_html__('-- None --', 'dd_builder'),
			'fade'			=> esc_html__('Fade', 'et_builder'),
			'slidedown' 	=> esc_html__('Slide Down', 'et_builder'),
			'slideup' 		=> esc_html__('Slide Up', 'et_builder'),
			'slideleft' 	=> esc_html__('Slide Left', 'et_builder'),
			'slideright' 	=> esc_html__('Slide Right', 'et_builder'),
			'zoom'			=> esc_html__('Zoom', 'et_builder'),
		);
	}
	public function get_effect_fields($toggle_slug, $show_if, $default_in = 'none', $default_out = 'none', $default_ms = '1000ms', $only_in = false, $prefix = '') {
		if ( $prefix !== '')
			$prefix .= '_';
		$fields = array(
			"{$prefix}effect_in" => array(
				'label'        	=> esc_html__( 'Enter Effect', 'dd_builder' ),
				'type'         	=> 'select',
				'options'       => $this->get_effect_options(),
				'default'		=> $default_in,
				'tab_slug'		=> 'advanced',
				'toggle_slug'  	=> $toggle_slug,			
			),
		);
		$fields["{$prefix}effect_in_intensity"] = array(
			'label'        		=> sprintf( '%1$s %2$s', esc_html__( 'Enter Effect', 'dd_builder' ), esc_html__( 'Animation Intensity', 'et_builder' )),
			'type'         		=> 'range',
			'default'			=> '100%',
			'fixed_unit'		=> '%',
			'show_if_not'		=> array( "{$prefix}effect_in" => 'none' ),
			'tab_slug'			=> 'advanced',
			'toggle_slug'  		=> $toggle_slug,			
		);
		if (!$only_in) {
			$fields["{$prefix}effect_out"] = array(
				'label'        	=> esc_html__( 'Exit Effect', 'dd_builder' ),
				'type'        	=> 'select',
				'options'       => $this->get_effect_options(),
				'default'		=> $default_out,
				'tab_slug'		=> 'advanced',
				'toggle_slug'  	=> $toggle_slug,				
			);
			$fields["{$prefix}effect_out_intensity"] = array(
				'label'        		=> sprintf( '%1$s %2$s', esc_html__( 'Exit Effect', 'dd_builder' ), esc_html__( 'Animation Intensity', 'et_builder' )),
				'type'         		=> 'range',
				'default'			=> '100%',
				'fixed_unit'		=> '%',
				'show_if_not'		=> array( "{$prefix}effect_out" => 'none' ),
				'tab_slug'			=> 'advanced',
				'toggle_slug'  		=> $toggle_slug,			
			);
		}
		$fields["{$prefix}effect_opacity"] = array(
			'label'        	=> esc_html__( 'Opacity', 'et_builder' ),
			'type'          => 'yes_no_button',
			'options'      	=> $this->get_yes_no_button_options(),
			'default'		=> 'off',
			'show_if_not'	=> array("{$prefix}effect_in" => array('none', 'fade' )),
			'tab_slug'		=> 'advanced',
			'toggle_slug'  	=> $toggle_slug,			
		);
		$fields["{$prefix}effect_opacity_intensity"] = array(
			'label'        	=> sprintf( '%1$s %2$s', esc_html__( 'Opacity', 'et_builder' ), esc_html__( 'Intensity', 'dd_builder' ) ),
			'type'         		=> 'range',
			'default'			=> '100%',
			'fixed_unit'		=> '%',
			'show_if_not'	=> array( "{$prefix}effect_opacity" => 'off' ),
			'tab_slug'		=> 'advanced',
			'toggle_slug'  	=> $toggle_slug,			
		);
		$fields["{$prefix}effect_ms"] = array(
				'label'         => esc_html__( 'Duration', 'dd_builder' ),
				'type'          => 'range',
				'default'       => $default_ms,
				'default_unit'  => 'ms',
				'tab_slug'		=> 'advanced',
				'toggle_slug'   => $toggle_slug,				
		);
		if ($show_if) {
			$fields["{$prefix}effect_in"]['show_if'] = $show_if;
			$fields["{$prefix}effect_out"]['show_if'] = $show_if;
			$fields["{$prefix}effect_in_intensity"]['show_if'] = $show_if;
			$fields["{$prefix}effect_out_intensity"]['show_if'] = $show_if;
			$fields["{$prefix}effect_opacity"]['show_if'] = $show_if;
			$fields["{$prefix}effect_ms"]['show_if'] = $show_if;
		}
		return $fields;
	}

	public function get_font_fields( $ul, $ol, $quote, $toggle ) {
		return array(
			"{$ul}_type"             => array(
				'label'            => esc_html__( 'Unordered List Style Type', 'et_builder' ),
				'description'      => esc_html__( 'This setting adjusts the shape of the bullet point that begins each list item.', 'et_builder' ),
				'type'             => 'select',
				'options'          => array(
					'disc'   => et_builder_i18n( 'Disc' ),
					'circle' => et_builder_i18n( 'Circle' ),
					'square' => et_builder_i18n( 'Square' ),
					'none'   => et_builder_i18n( 'None' ),
				),
				'priority'         => 80,
				'default'          => 'disc',
				'tab_slug'         => 'advanced',
				'toggle_slug'      => $toggle,
				'sub_toggle'       => 'ul',
				'mobile_options'   => true,
			),
			"{$ul}_position"         => array(
				'label'            => esc_html__( 'Unordered List Style Position', 'et_builder' ),
				'description'      => esc_html__( 'The bullet point that begins each list item can be placed either inside or outside the parent list wrapper. Placing list items inside will indent them further within the list.', 'et_builder' ),
				'type'             => 'select',
				'options'          => array(
					'outside' => et_builder_i18n( 'Outside' ),
					'inside'  => et_builder_i18n( 'Inside' ),
				),
				'priority'         => 85,
				'default'          => 'outside',
				'tab_slug'         => 'advanced',
				'toggle_slug'      => $toggle,
				'sub_toggle'       => 'ul',
				'mobile_options'   => true,
			),
			"{$ul}_item_indent"      => array(
				'label'            => esc_html__( 'Unordered List Item Indent', 'et_builder' ),
				'description'      => esc_html__( 'Increasing indentation will push list items further towards the center of the text content, giving the list more visible separation from the the rest of the text.', 'et_builder' ),
				'type'             => 'range',
				'tab_slug'         => 'advanced',
				'toggle_slug'      => $toggle,
				'sub_toggle'       => 'ul',
				'priority'         => 90,
				'default'          => '0px',
				'default_unit'     => 'px',
				'allowed_units'    => array( '%', 'em', 'rem', 'px', 'cm', 'mm', 'in', 'pt', 'pc', 'ex', 'vh', 'vw' ),
				'range_settings'   => array(
					'min'  => '0',
					'max'  => '100',
					'step' => '1',
				),
				'mobile_options'   => true,
			),
			"{$ol}_type"             => array(
				'label'            => esc_html__( 'Ordered List Style Type', 'et_builder' ),
				'description'      => esc_html__( 'Here you can choose which types of characters are used to distinguish between each item in the ordered list.', 'et_builder' ),
				'type'             => 'select',
				'options'          => array(
					'decimal'              => 'decimal',
					'armenian'             => 'armenian',
					'cjk-ideographic'      => 'cjk-ideographic',
					'decimal-leading-zero' => 'decimal-leading-zero',
					'georgian'             => 'georgian',
					'hebrew'               => 'hebrew',
					'hiragana'             => 'hiragana',
					'hiragana-iroha'       => 'hiragana-iroha',
					'katakana'             => 'katakana',
					'katakana-iroha'       => 'katakana-iroha',
					'lower-alpha'          => 'lower-alpha',
					'lower-greek'          => 'lower-greek',
					'lower-latin'          => 'lower-latin',
					'lower-roman'          => 'lower-roman',
					'upper-alpha'          => 'upper-alpha',
					'upper-greek'          => 'upper-greek',
					'upper-latin'          => 'upper-latin',
					'upper-roman'          => 'upper-roman',
					'none'                 => 'none',
				),
				'priority'         => 80,
				'default'          => 'decimal',
				'tab_slug'         => 'advanced',
				'toggle_slug'      => $toggle,
				'sub_toggle'       => 'ol',
				'mobile_options'   => true,
			),
			"{$ol}_position"         => array(
				'label'            => esc_html__( 'Ordered List Style Position', 'et_builder' ),
				'description'      => esc_html__( 'The characters that begins each list item can be placed either inside or outside the parent list wrapper. Placing list items inside will indent them further within the list.', 'et_builder' ),
				'type'             => 'select',
				'options'          => array(
					'inside'  => et_builder_i18n( 'Inside' ),
					'outside' => et_builder_i18n( 'Outside' ),
				),
				'priority'         => 85,
				'default'          => 'inside',
				'tab_slug'         => 'advanced',
				'toggle_slug'      => $toggle,
				'sub_toggle'       => 'ol',
				'mobile_options'   => true,
			),
			"{$ol}_item_indent"      => array(
				'label'            => esc_html__( 'Ordered List Item Indent', 'et_builder' ),
				'description'      => esc_html__( 'Increasing indentation will push list items further towards the center of the text content, giving the list more visible separation from the the rest of the text.', 'et_builder' ),
				'type'             => 'range',
				'tab_slug'         => 'advanced',
				'toggle_slug'      => $toggle,
				'sub_toggle'       => 'ol',
				'priority'         => 90,
				'default'          => '0px',
				'default_unit'     => 'px',
				'allowed_units'    => array( '%', 'em', 'rem', 'px', 'cm', 'mm', 'in', 'pt', 'pc', 'ex', 'vh', 'vw' ),
				'range_settings'   => array(
					'min'  => '0',
					'max'  => '100',
					'step' => '1',
				),
				'mobile_options'   => true,
			),
			"{$quote}_border_weight" => array(
				'label'            => esc_html__( 'Blockquote Border Weight', 'et_builder' ),
				'description'      => esc_html__( 'Block quotes are given a border to separate them from normal text. You can increase or decrease the size of that border using this setting.', 'et_builder' ),
				'type'             => 'range',
				'tab_slug'         => 'advanced',
				'toggle_slug'      => $toggle,
				'sub_toggle'       => 'quote',
				'priority'         => 85,
				'default'          => '5px',
				'default_unit'     => 'px',
				'default_on_front' => '',
				'allowed_units'    => array( 'em', 'rem', 'px', 'cm', 'mm', 'in', 'pt', 'pc', 'ex', 'vh', 'vw' ),
				'range_settings'   => array(
					'min'  => '0',
					'max'  => '100',
					'step' => '1',
				),
				'mobile_options'   => true,
				'sticky'           => true,
				'hover'            => 'tabs',
			),
			"{$quote}_border_color"  => array(
				'label'           => esc_html__( 'Blockquote Border Color', 'et_builder' ),
				'description'     => esc_html__( 'Block quotes are given a border to separate them from normal text. Pick a color to use for that border.', 'et_builder' ),
				'type'            => 'color-alpha',
				'custom_color'    => true,
				'tab_slug'        => 'advanced',
				'toggle_slug'     => $toggle,
				'sub_toggle'      => 'quote',
				'field_template'  => 'color',
				'priority'        => 90,
				'mobile_options'  => true,
				'sticky'          => true,
				'hover'           => 'tabs',
			),
		);
	}

	public function get_layouts() {
		if (empty(self::$layouts)) {
			$posts = get_posts(array('post_type' => 'et_pb_layout', 'numberposts' => -1)); 
			self::$layouts[] = __('-- Select --', 'dd_builder');
			foreach ($posts as $post) {
				if ( strpos($post->post_content, 'ddbuilder_menu') !== false )
					continue;		
				self::$layouts[$post->ID] = $post->post_title;		
			}
		}
		return self::$layouts;
	}

	public function get_popup_fields($settings) {
		$preview = array();
		if ( isset( $settings['preview'])) {
			$preview['popup_preview'] = array( 
				'label'            	=> esc_html__( 'Preview in the Builder', 'dd_builder' ),
				'description'		=> esc_html__( 'Toggle this field to temporarily preview the popup.', 'dd_builder' ),
				'type'          	=> 'yes_no_button',
				'options'      		=> $this->get_yes_no_button_options(),
				'default'			=> 'off',
				'show_if' 			=> isset($settings['show_if_custom']) ? $settings['show_if_custom'] : $settings['show_if'],
				'toggle_slug'     	=> $settings['toggle_slug'],
			);
		}
		$content_fields = array (
			'popup_hover' => array(
				'label'         => esc_html__( 'Show On Hover', 'dd_builder' ),
				'description'   => esc_html__( 'Here you can choose whether te popup should be displayed on hover as well.', 'dd_builder' ),
				'type'          => 'yes_no_button',
				'options'      	=> $this->get_yes_no_button_options(),
				'default'		=> 'off',
				'show_if' 		=> isset($settings['show_if_hover']) ? $settings['show_if_hover'] : (
					isset($settings['show_if_custom']) ? $settings['show_if_custom'] : $settings['show_if']
				),
				'toggle_slug'   => $settings['toggle_slug'],		
			),
			'popup_close' => array(
				'label'       	=> esc_html__( 'Show Close Icon', 'dd_builder' ),
				'description' 	=> esc_html__( 'Here can you choose whether or not a close icon should be also used to dismiss the popup.', 'dd_builder' ),
				'type'          => 'yes_no_button',
				'options'      	=> $this->get_yes_no_button_options(),
				'default'		=> 'off',
				'show_if' 		=> isset($settings['show_if_custom']) ? $settings['show_if_custom'] : $settings['show_if'],
				'toggle_slug'   => $settings['toggle_slug'],				
			),
			'popup_close_font_icon' => array(
				'label'        	=> esc_html__( 'Close Icon', 'dd_builder' ),
				'type'         	=> 'select_icon',
				'class'         => array( 'et-pb-font-icon' ),
				'default'		=> '&#x4d;||divi||400',
				'show_if' 		=> isset($settings['show_if_custom']) ? $settings['show_if_custom'] : $settings['show_if'],
				'show_if_not' 	=> array('popup_close' => 'off'),
				'toggle_slug'   => $settings['toggle_slug'],
				'hover'			=> 'tabs',				
			),
			'popup_close_click' => array(
				'label'       		=> esc_html__( 'Close On Icon Click Only', 'dd_builder' ),
				'type'          	=> 'yes_no_button',
				'options'      		=> $this->get_yes_no_button_options(),
				'default'			=> 'off',
				'show_if' 			=> isset($settings['show_if_custom']) ? $settings['show_if_custom'] : $settings['show_if'],
				'show_if_not' 		=> array('popup_close' => 'off'),
				'mobile_options'  	=> true,
				'toggle_slug'   	=> $settings['toggle_slug'],				
			),
			'popup_disable_page_scroll' => array(
				'label'       	=> esc_html__( 'Disable Page Scroll', 'dd_builder' ),
				'description' 	=> esc_html__( 'Here can you choose whether or not the page scroll should be disabled when the popup is open.', 'dd_builder' ),
				'type'          => 'yes_no_button',
				'options'      	=> $this->get_yes_no_button_options(),
				'default'		=> 'off',
				'show_if' 		=> isset($settings['show_if_custom']) ? $settings['show_if_custom'] : $settings['show_if'],
				'toggle_slug'   => $settings['toggle_slug'],				
			),
			'__popup'	=> array(
				'type'                => 'computed',
				'computed_callback'   => array( 'DonDiviBuilderHelper', 'render_divi_layout' ),
				'computed_depends_on' => array(
					'divi_layout',
				),
				'computed_minimum' => array(
					'divi_layout',
				),
			),
		);
		$design_fields = array(
			'popup_location' => array( 
				'label'        		=> esc_html__( 'Location', 'et_builder' ),
				'type'         		=> 'select',
				'options'       	=> ddbuilder_get_location_options(),
				'default' 			=> 'center',
				'show_if' 			=> isset($settings['show_if_custom']) ? $settings['show_if_custom'] : $settings['show_if'],
				'mobile_options'  	=> true,
				'tab_slug'      	=> 'advanced',
				'toggle_slug'   	=> $settings['advanced_slug'],				
			),
			'popup_max_width' => array( 
				'label'       		=> esc_html__( 'Popup Max Width', 'dd_builder' ),
				'type'         		=> 'range',				
				'range_settings' 	=> array(
					'min_limit'  	=> 10,
					'step'			=> 1,
				),	
				'fixed_unit'		=> '%',
				'default'			=> '80%',
				'show_if' 			=> isset($settings['show_if_custom']) ? $settings['show_if_custom'] : $settings['show_if'],
				'mobile_options'  	=> true,
				'tab_slug'     		=> 'advanced',
				'toggle_slug'   	=> $settings['advanced_slug'],				
			),
			'popup_max_height' => array( 
				'label'        		=> esc_html__( 'Popup Max Height', 'dd_builder' ),
				'type'         		=> 'range',			
				'range_settings' 	=> array(
					'min_limit'  	=> 10,
					'step'			=> 1,
				),	
				'fixed_unit'		=> '%',
				'default'			=> '80%',	
				'mobile_options'  	=> true,
				'show_if' 			=> isset($settings['show_if_custom']) ? $settings['show_if_custom'] : $settings['show_if'],
				'tab_slug'      	=> 'advanced',
				'toggle_slug'   	=> $settings['advanced_slug'],						
			),
			'content_background_color' => array(
				'label'             => esc_html__( 'Background Color', 'et_builder' ),
				'type'              => 'color-alpha',
				'default' 			=> '#ffffff',
				'mobile_options'  	=> true,
				'show_if'			=> isset($settings['show_if_custom']) ? $settings['show_if_custom'] : $settings['show_if'],
				'tab_slug'          => 'advanced',
				'toggle_slug'       => $settings['advanced_slug'],
			),	
			'overlay_color' => array(
				'label'           	=> esc_html__( 'Overlay Color', 'dd_builder' ),
				'type'            	=> 'color-alpha',	
				'default'			=> 'rgba(0,0,0,0.4)',
				'mobile_options'  	=> true,
				'show_if' 			=> isset($settings['show_if_custom']) ? $settings['show_if_custom'] : $settings['show_if'],
				'tab_slug'        	=> 'advanced',
				'toggle_slug'   	=> $settings['advanced_slug'],	
			),
			'popup_close_text_color' => array(
				'label'           	=> esc_html__( 'Close Icon Color', 'dd_builder' ),
				'type'            	=> 'color-alpha',
				'default'			=> '#666666',
				'mobile_options'  	=> true,	
				'show_if' 			=> isset($settings['show_if_custom']) ? $settings['show_if_custom'] : $settings['show_if'],
				'show_if_not'		=> array( 'popup_close' => 'off'),
				'tab_slug'        	=> 'advanced',
				'toggle_slug'   	=> $settings['advanced_slug'],	
			),	
			'popup_close_font_size' => array(
				'label'           	=> esc_html__( 'Close Icon Font Size', 'dd_builder' ),
				'type'            	=> 'range',			
				'default'  			=> '40px',
				'fixed_unit'		=> 'px',
				'mobile_options'  	=> true,
				'show_if' 			=> isset($settings['show_if_custom']) ? $settings['show_if_custom'] : $settings['show_if'],
				'show_if_not'		=> array( 'popup_close' => 'off'),
				'tab_slug'        	=> 'advanced',
				'toggle_slug'     	=> $settings['advanced_slug'],
			),
		);
		$advanced_fields = array(
			'popup_id' => array(
				'label'           	=> esc_html__( 'Popup ID', 'dd_builder' ),
				'type'            	=> 'text',
				'show_if'			=> isset($settings['show_if_custom']) ? $settings['show_if_custom'] : $settings['show_if'],
				'tab_slug'        	=> 'custom_css',
				'toggle_slug'     	=> 'classes',
				'option_class'    	=> 'et_pb_custom_css_regular',
			),
		);
		return array_merge( $preview, $content_fields, $design_fields, $advanced_fields );
	}

	public function get_positions() {
		$positions = array(
			'top' 		=> esc_html__( 'Top', 'et_builder' ),
			'right'  	=> esc_html__( 'Right', 'et_builder' ),
			'bottom'  	=> esc_html__( 'Bottom', 'et_builder' ),
			'left'  	=> esc_html__( 'Left', 'et_builder' ),					
		);
		return $positions;
	}

	public function get_url_fields($toggle_slug, $link_types, $rel_values) {
		return array(
			'url' => array(
				'label'           	=> esc_html__( 'URL', 'et_builder' ),
				'description'     	=> esc_html__( 'Enter your destination URL here.', 'dd_builder' ),
				'type'            	=> 'text',
				'show_if'			=> array('link_type' => $link_types),
				'toggle_slug'     	=> $toggle_slug,
				'dynamic_content' 	=> 'url',				
			),
			'url_new_window' => array(
				'label'           	=> esc_html__( 'URL Opens', 'dd_builder' ),
				'description'     	=> esc_html__( 'Here you can choose whether or not your link opens in a new window', 'et_builder' ),
				'type'            	=> 'select',
				'options'         	=> array(
					'off' => esc_html__( 'In The Same Window', 'et_builder' ),
					'on'  => esc_html__( 'In The New Tab', 'et_builder' ),
				),
				'show_if'			=> array('link_type' => $link_types),
				'default'  			=> 'off',
				'toggle_slug'     	=> $toggle_slug,
			),	
			'link_rel'  => array(
				'label'           	=> esc_html__( 'Link Relationship', 'et_builder' ),
				'type'            	=> 'multiple_checkboxes',
				'options'         	=> $rel_values,
				'show_if'			=> array('link_type' => $link_types),
				'description'     	=> et_get_safe_localization( __( "Specify the value of your link's <em>rel</em> attribute. The <em>rel</em> attribute specifies the relationship between the current document and the linked document.<br><strong>Tip:</strong> Search engines can use this attribute to get more information about a link.", 'et_builder' ) ),
				'tab_slug'        	=> 'custom_css',
				'toggle_slug'     	=> 'attributes',
			),
		);
	}

	public function get_yes_no_button_options() {
		return array(
			'off' => et_builder_i18n( 'No' ),
			'on'  => et_builder_i18n( 'Yes' ),
		);
	}

}

class DonDiviBuilderHelper {

	private static $instance;

	public static function get() {
		if ( empty( self::$instance ) ) {
			self::$instance = new DonDiviBuilderHelper();
		}

		return self::$instance;
	}

	public static function render_divi_layout( $args = array() ) {
		global $et_pb_rendering_column_content;
		
		$defaults = array(
			'internal' => true,
		);
		$args = wp_parse_args( $args, $defaults );
		
		$layout_id = '';
		if ( !empty( $args['layout_id'] ) )
			$layout_id = $args['layout_id'];
		else if ( !empty( $args['divi_layout'] ) )
			$layout_id = $args['divi_layout'];
		else return $layout_id;
		
		if ( $args['internal'] ) $et_pb_rendering_column_content = true;
		//ET_Builder_Element::clean_internal_modules_styles();
		
		$rendered_layout = do_shortcode( get_the_content( null, false, $layout_id ) );
		
		if ($args['internal']) {
			$rendered_layout .= '<style class="ddb-layout-style"> ' . et_core_esc_previously( ET_Builder_Element::get_style() ) . ' </style>';
			$et_pb_rendering_column_content = false;
		}
		//ET_Builder_Element::clean_internal_modules_styles( false );

		return $rendered_layout;
	}

	public static function render_wp_nav_menu( $args = array(), $conditional_tags = array(), $current_page = array() ) {
		$defaults     = array(
			'frontend' 			=> false,
			'wp_accordion'     	=> 'off',
			'wp_collapse'     	=> 'off',
			'wp_menu_id'    	=> '',
		);
		$args      = wp_parse_args( $args, $defaults );
		$menu_args = array(
			'container'     => '',
			'link_before'	=> '<span style="flex:1">',
			'link_after'	=> '</span>',
			'link_class'	=> 'ddb-wp-item ddb-f ddb-dark ddb-p-xs',
			'menu'			=> (int) $args['wp_menu_id'],
			'menu_class'	=> '',
			'echo'          => false,
		);
		
		if ($args['wp_collapse'] === 'on') {
			$menu_args['li_collapse_class'] = $args['wp_accordion'] === 'on' ? 'ddb-collapsible ddb-accordion' : 'ddb-collapsible';
			$menu_args['link_collapse_class'] = $menu_args['link_class'] . ' ddb-collapse';
			if ($args['frontend']) {
				$helper = DonDiviBuilderHelper::get();
				$menu_args['link_after'] = '</span>' . $helper->render_icon( array( 
					'module' 	=> $args['module_instance'], 
					'icon_attr' => 'wp_menu_icon', 
					'selector' 	=> '%%order_class%% span.ddb-collapse',
					'hover_selector' => '%%order_class%% .ddb-collapse.active > span.ddb-collapse',
					'attrs' 	=> array( 'class' => 'ddb-collapse ddb-click notranslate'),
					'custom' 	=> true
				) );
			} else	
			    $menu_args['link_after'] = sprintf('</span><span class="ddb-icon ddb-collapse notranslate">%s</span>', esc_html( et_pb_extended_process_font_icon( $args['wp_menu_icon'] )));
		}

		return wp_nav_menu( apply_filters( 'dondivi_wp_nav_menu_args', $menu_args ) );
	}

	public function get_flex_alignment($property, $direction, $main_axis, $cross_axis) {
		if ( ($property === 'justify' && $direction !== 'column' && $direction !== 'column-reverse') || ($property === 'align' && ( $direction === 'column' || $direction === 'column-reverse'))) {
			switch ($main_axis) {
				case 'flex-start':
				case 'top':
				case 'left':
				case 'force_left': return $direction === 'column-reverse' || $direction === 'row-reverse' ? 'flex-end' : 'flex-start';
				case 'justified': return $direction === 'column' || $direction === 'column-reverse' ? 'center' : 'space-between';
				case 'flex-end':
				case 'bottom':
				case 'right': return $direction === 'column-reverse' || $direction === 'row-reverse' ? 'flex-start' : 'flex-end';
				default:
				return 'center';
			} 
		} 
		if ( ($property === 'align' && $direction !=='column') || ($property === 'justify' && $direction === 'column') )
			return $cross_axis;
		return 'center';
	}

	public function get_responsive_classnames($field_values, $responsive, $base_class, $values) {
		$classes = array();
		if ( !$responsive) {
			if (in_array($field_values['desktop'], $values))  $classes []= $base_class;
		} else {
			if (in_array($field_values['desktop'], $values))  $classes []= "$base_class-d";
			if (in_array($field_values['tablet'], $values))  $classes []= "$base_class-t";
			if (in_array($field_values['phone'], $values))  $classes []= "$base_class-p";
		}		
		return implode(' ', $classes);
	}
 
	public function get_custom_button($module, $button_name, $button_text, $button_esc_text, $button_classes, $wrapper_classes = '', $has_url_field = false, $prefix = '') {
		$multi_view		= et_pb_multi_view_options( $module );
		$button_url 	= !$has_url_field ? '#' : $module->props["{$prefix}cc_button_url"]; 
		$button_rel 	= isset($module->props[$button_name . '_rel']) ? $module->props[$button_name . '_rel'] : '';
		
		$icon_values = et_pb_responsive_options()->get_property_values( $module->props, "{$button_name}_icon" );
		$icon        = isset( $icon_values['desktop'] ) ? $icon_values['desktop'] : '';
		$icon_tablet = isset( $icon_values['tablet'] ) ? $icon_values['tablet'] : '';
		$icon_phone  = isset( $icon_values['phone'] ) ? $icon_values['phone'] : '';
		$visibility = array(
			"$button_text" => '__not_empty',
		);
		if ( $has_url_field ) $visibility["{$prefix}cc_button_url"] = '__not_empty';

		$button = $module->render_button(
			array(
				'button_classname'    => array( $button_classes ),
				'button_custom'       => $module->props['custom_' . $button_name],
				'button_rel'          => $button_rel,
				'button_text'         => !empty($button_esc_text) ? $button_esc_text : $module->props[$button_text],
				'button_text_escaped' => !empty($button_esc_text),
				'button_url'          => $button_url,
				'custom_icon'         => $icon,
				'custom_icon_tablet'  => $icon_tablet,
				'custom_icon_phone'   => $icon_phone,
				'has_wrapper'			=> false,
				'url_new_window'      => $has_url_field ? $module->props["{$prefix}cc_button_new_window"] : '',
				'display_button'      => ( (!$has_url_field || '' !== $button_url) && $multi_view->has_value( $button_text ) ),
				'multi_view_data'     => $multi_view->render_attrs(
					array(
						'content'    => '{{' . $button_text .'}}',
						'visibility' => $visibility,
					)
				),
			)
		);

		if ($button) $button = sprintf('<div class="et_pb_button_module_wrapper%2$s">%1$s</div>', 
			$button,
			$wrapper_classes ? ' ' . $wrapper_classes : '' 
		);

		$this->maybe_late_icons_asset( $icon_values );
		
		return $button;
	}

	public function late_icons_all( $assets, $assets_args, $et_dynamic_assets ) {	
		if ( isset( $assets['et_icons_all'] ) ) 
			return $assets;	
		
		$assets_prefix = et_get_dynamic_assets_path();
		$assets['et_icons_all'] = array(
			'css' => "{$assets_prefix}/css/icons_all.css",
		);
		return $assets;
	}

	public function late_icons_fa( $assets, $assets_args, $et_dynamic_assets ) {	
		if ( isset( $assets['et_icons_fa'] ) ) 
			return $assets;	
		
		$assets_prefix = et_get_dynamic_assets_path();
		$assets['et_icons_fa'] = array(
			'css' => "{$assets_prefix}/css/icons_fa_all.css",
		);
		return $assets;
	}

	public function maybe_late_icons_asset( $icon_values ) {
		foreach( $icon_values as $key => $value ){
			if (strpos( $value, '||fa||') !== false ) {
				add_filter( 'et_late_global_assets_list', array( $this, 'late_icons_fa' ), 10, 3 );
			} else if (strpos( $value, '||divi||') !== false ) {
				add_filter( 'et_late_global_assets_list', array( $this, 'late_icons_all' ), 10, 3 );
			}
		}
	}

	public function render_custom_content($module, $args = array()) {
		$defaults = array(
			'prefix' => '',
			'header_level' => 'h4',
			'slug' => 'content',
			'selector' => '%%order_class%%',
			'waypoints' => false,
		);
		$args = wp_parse_args( $args, $defaults );
		$prefix = $args['prefix'] !== '' ? $args['prefix'] . '_' : '';
		$selector = $args['selector'] . " .ddb-custom-{$args['slug']}";

		$multi_view = et_pb_multi_view_options( $module );

		$title = DonDiviBuilderHelper::render_title( $module, "{$prefix}cc_title", "{$prefix}title", $args['header_level'], "ddb-{$args['slug']}-title", true, $prefix );

		$body = '';
		if ( $multi_view->has_value( "{$prefix}content" ) ) {
			$body = $multi_view->render_element(
				array(
					'tag'     => 'div',
					'content' => "{{{$prefix}content}}",
					'attrs'   => array(
						'class' => "ddb-{$args['slug']}-body ddb-pb",
					),
					'visibility' => array( "{$prefix}content" => '__not_empty' ),
				)
			);
		}
		$image = '';
		if ('icon' === $module->props["{$prefix}cc_icon_image"]) {
			$icon_attrs = array( 'class' => "ddb-{$args['slug']}-image-icon ddb-block" ); // we need ddb-block (or inline-block) so the waypoint animation works
			if ( $args['waypoints'] ) {
				$waypoints_prefix = isset($args['effects_prefix']) ? $args['effects_prefix'] : $prefix;
				$this->set_effect_attrs($icon_attrs, $module->props, array( 'lazy_loading' => true, 'prefix' => $waypoints_prefix ) );
			}
			$image = $this->render_icon( array( 
				'module' => $module, 
				'icon_attr' => "{$prefix}cc_icon", 
				'color_attr' => "{$prefix}cc_icon_color", 
				'size_attr' => "{$prefix}cc_icon_width", 
				'selector' => $selector . " .ddb-{$args['slug']}-image-icon", 
				'attrs' => $icon_attrs
			) );
		}
		else if ( 'image' === $module->props["{$prefix}cc_icon_image"] ) {
			$image_attrs = array( 'class' => "ddb-{$args['slug']}-image-icon ddb-bss" );
			if ( $args['waypoints'] ) {
				$waypoints_prefix = isset($args['effects_prefix']) ? $args['effects_prefix'] : $prefix;
				$this->set_effect_attrs($image_attrs, $module->props, array( 'lazy_loading' => true, 'prefix' => $waypoints_prefix ) );
			}
			$image = $this->render_image( array( 
				'module' => $module,
				'alt_attr' => "{$prefix}cc_alt", 
				'img_attr' => "{$prefix}cc_image",
				'attrs' => $image_attrs
			) );
			if ( $image !== '' ) {
				$module->generate_styles(
					array(
						'base_attr_name' => "{$prefix}cc_image_width",
						'selector'       => $selector . " .ddb-{$args['slug']}-image-icon",
						'css_property'   => array( 'min-width', 'width' ),
						'render_slug'    => $module->slug,
					)
				);
			}  
		}
		if ($image !== '') {
			$pos = et_pb_responsive_options()-> get_property_values($module->props, "{$prefix}cc_icon_placement", $module->props["{$prefix}cc_icon_placement"], true);
			$hali = et_pb_responsive_options()-> get_property_values($module->props, "{$prefix}cc_icon_alignment", $module->props["{$prefix}cc_icon_alignment"], true);
			$vali = et_pb_responsive_options()-> get_property_values($module->props, "{$prefix}cc_icon_vertical_alignment", $module->props["{$prefix}cc_icon_vertical_alignment"], true);
			foreach ( et_pb_responsive_options()->get_modes() as $device ) {
				if ( $hali[$device] !== '' && $pos[$device] !== 'row' )
					$this->set_style( 
						$module->slug, $selector . " .ddb-{$args['slug']}-image-icon", $this->set_margin_alignment( $hali[$device] ), substr( $device, 0, 1 ) 
					);
				if ( $pos[$device] === 'row' ) {
					$this->set_style( 
						$module->slug, $selector, sprintf('overflow-x:auto;align-items:%s;', $vali[$device]), substr( $device, 0, 1 ) 
					);
				}
			}
		}
		$module->generate_styles(
			array(
				'base_attr_name' => "{$prefix}cc_icon_placement",
				'selector'       => $selector,
				'css_property'   => 'flex-direction',
				'render_slug'    => $module->slug,
			)
		);
		$module->generate_styles(
			array(
				'base_attr_name' => "{$prefix}cc_gap",
				'selector'       => $selector,
				'css_property'   => 'gap',
				'render_slug'    => $module->slug,
			)
		);
		$button = DonDiviBuilderHelper::get_custom_button($module, "{$prefix}button", "{$prefix}cc_button_text", '' , "ddb-{$args['slug']}-button", '', true, $prefix);

		return sprintf('%3$s<div class="ddb-content">%1$s%2$s%4$s</div>', $title, $body, $image, $button);
	}
	
	public function render_icon( $icon_args ) {
		$defaults = array(
			'attrs' => array(),
			'custom' => false,
			'hover_selector' => '',
		);
		$args = wp_parse_args( $icon_args, $defaults );
		
		if ( !isset($args['content_hover'] ) ) 
			$args['content_hover'] = $args['hover_selector'];

		$module = $args['module'];
		$attrs = $args['attrs'];

		$icon_values = 	et_pb_responsive_options()-> get_property_values( $module->props, $args['icon_attr'], $module->props[$args['icon_attr']], true );
		$icon_values['hover'] = et_pb_hover_options()->get_value( $args['icon_attr'], $module->props, $icon_values['desktop']);

		$attrs['tabindex'] = '0';
		$attrs['aria-label'] = 'Icon';
		if ( isset($attrs['class']) ) 
			$attrs['class'] .= ' ddb-icon notranslate';
		else $attrs['class'] = 'ddb-icon notranslate';

		if ( $args['custom'] ) {
			$multi_value = false;
			$multi_attrs = array();  
			foreach ( $icon_values as $mode => $value ) {
				if ( $icon_values['desktop'] !== $value || $icon_values['hover'] !== $value ) $multi_value = true;
				$multi_attrs["data-{$mode}"] = esc_attr( et_pb_extended_process_font_icon( $value ) );
			}
			if ( $multi_value ) {
				$attrs['class'] .= ' ddb-multi-value ddb-hide-tablet ddb-hide-phone';
				$attrs = array_merge( $attrs, $multi_attrs );
			}
			$attrs_output  = '';
			foreach( $attrs as $attr => $value){
				$attrs_output .= ' '. $attr . '="' . $value . '"';
			}
			$icon_output = sprintf('<span%2$s>%1$s</span>', $multi_attrs['data-desktop'], $attrs_output );
			
		} else {
			$multi_view = et_pb_multi_view_options( $module );
			$icon_output = $multi_view->render_element(
				array(
					'content' => '{{' . $args['icon_attr'] .'}}',
					'hover_selector' => $args['content_hover'],
					'attrs'   => $attrs,
				)
			);
		}
		$module->generate_styles(
			array(
				'utility_arg'    => 'icon_font_family',
				'render_slug'    => $module->slug,
				'base_attr_name' => $args['icon_attr'],
				'important'      => true,
				'selector'       => $args['selector'],
				'hover_selector' => $args['content_hover'],
				'processor'      => array(
					'ET_Builder_Module_Helper_Style_Processor',
					'process_extended_icon',
				),
			)
		);

		$this->maybe_late_icons_asset( $icon_values );

	    if ( isset( $args['color_attr'] ) ) {
			$module->generate_styles(
				array(
					'base_attr_name' => $args['color_attr'],
					'selector'       => $args['selector'],
					'css_property'   => 'color',
					'render_slug'    => $module->slug,
					'type'           => 'color',
					'hover_selector' => $args['hover_selector'],
				)
			);
		}
		if ( isset( $args['size_attr'] ) ) {
			$module->generate_styles(
				array(
					'base_attr_name' => $args['size_attr'],
					'selector'       => $args['selector'],
					'css_property'   => 'font-size',
					'render_slug'    => $module->slug,
					'type'           => 'range',
					'hover_selector' => $args['hover_selector'],
				)
			);
		}
		return $icon_output;
	}

	public function render_image( $img_args ) {	
		$defaults = array(
			'alt_attr' => '',
			'alt_value' => '',
			'attrs' => array(),
			'custom' => false,
			'default_image' => ''
		);
		$args 		= wp_parse_args( $img_args, $defaults );
		$module 	= $args['module'];
		$multi_view = et_pb_multi_view_options( $module );
		$attrs 		= $args['attrs'];
		if (!( ( isset( $args['img_attr'] ) && $multi_view->has_value( $args['img_attr'] ) ) || isset($args['img_src']) ) && $args['default_image'] === '' ) return '';
        $img_values = array();
		if ( isset( $args['img_attr'] ) )  {
			$default_image = $module->props[$args['img_attr']] !== '' ? $module->props[$args['img_attr']] : $args['default_image'];
			$img_values = et_pb_responsive_options()-> get_property_values( $module->props, $args['img_attr'], $default_image, true );
			
			if ( et_pb_hover_options()->is_enabled( $args['img_attr'], $module->props ) ) 			
				$img_values['hover'] = et_pb_hover_options()->get_value( $args['img_attr'], $module->props );

			if ( et_pb_sticky_options()->is_enabled( $args['img_attr'], $module->props ) )	{
				$sticky_value = et_pb_sticky_options()->get_value( $args['img_attr'], $module->props );
				if ($sticky_value !== '') 
					$img_values['sticky'] = esc_attr( $sticky_value );
			}
		} else {
			$img_values['desktop'] = $args['img_src']; 
		}

		if ( $args['alt_attr'] !== '' && isset( $module->props[$args['alt_attr']] ) && $module->props[$args['alt_attr']] !== '' ) { 
			$attrs['alt'] = esc_attr( $module->props[$args['alt_attr']] );
		} else if ( $args['alt_value'] !== '' )
			$attrs['alt'] = esc_attr( $args['alt_value']);
		else {
			$attrs['alt'] = ddbuilder_get_image_alt( $img_values['desktop'] );            
		}
		
		if ( $args['custom'] ) {
			$multi_value = false;
			$multi_attrs = array();  
			foreach ( $img_values as $mode => $value ) {
				if ( $img_values['desktop'] !== $value || ( isset( $img_values['hover'] ) && $img_values['hover'] !== $value ) || isset( $img_values['sticky'] ) ) 
					$multi_value = true;
				$multi_attrs["data-{$mode}"] = $value;
				
				if ( $mode === 'desktop' && $args['alt_attr'] !== '' && isset( $module->props[$args['alt_attr']] ) && $module->props[$args['alt_attr']] !== '' ) { 
					$attrs['data-alt-desktop'] = esc_attr( $module->props[$args['alt_attr']] );
				} else if ( $mode === 'desktop' && $args['alt_value'] !== '' )
					$attrs['data-alt-desktop'] = esc_attr( $args['alt_value']);
				 else {
					$attrs["data-alt-{$mode}"] = ddbuilder_get_image_alt( $img_values[$mode] );            
				}
			}
			if ( $multi_value ) {
				!isset( $attrs['class'] ) ? $attrs['class'] = 'ddb-multi-value' : $attrs['class'] .= ' ddb-multi-value';
				if ( $img_values['desktop'] === '' ) $attrs['class'] .= ' ddb-hide-desktop';
				$attrs['class'] .= ' ddb-hide-tablet ddb-hide-phone';
				$attrs = array_merge( $attrs, $multi_attrs );
			} 
			$attrs['src'] = $img_values['desktop'];
		
			$attrs_output  = '';
			foreach( $attrs as $attr => $value) {
				$attrs_output .= ' '. $attr . '="' . $value  . '"';
			}
			return sprintf('<img%s/>', $attrs_output );
		} else {
			if ( isset( $args['img_attr'] ) ) {
				$attrs['src'] = $module->props[$args['img_attr']] !== '' ? '{{'. $args['img_attr'] . '}}' : $args['default_image'];
			} else $attrs['src'] = $img_values['desktop'];
	
			return $multi_view->render_element(
				array(	
					'tag'      => 'img',
					'attrs'    => $attrs,
				)
			);
		}
	}

	public function render_popup( $args ) {
		$attrs 			= $args['attrs'];
		$module 		= $args['module'];
		$slug			= $args['slug'];
		$multi_view		= et_pb_multi_view_options( $module );
		
		$popup_id 		= $module->get_module_order_class($module->slug) . '_popup';
		$popup_selector = '%%order_class%%_popup';
		
		$attrs['id'] 	= empty( $module->props['popup_id'] ) ? $popup_id : esc_attr( $module->props['popup_id'] );
		$attrs['class'] = "ddb-popup-c ddb-tl ddb-100 ddb-none et_pb_section et_pb_column_4_4 {$popup_id}";
		$close_click 	= et_pb_responsive_options()-> get_property_values( $module->props, 'popup_close_click', $module->props['popup_close_click'], true );
		$location 		= et_pb_responsive_options()-> get_property_values( $module->props, 'popup_location', $module->props['popup_location'], true );
		$popup_content 	= '';
		$popup_close 	= '';
		
		foreach ( et_pb_responsive_options()->get_modes() as $device ) {
			$location_array = explode( '_', $location[$device] );
			$location_y 	= isset( $location_array[0] ) ? $location_array[0] : 'center';
			$location_x 	= isset( $location_array[1] ) ? $location_array[1] : 'center';
			$this->set_style( 
				$module->slug, 
				$popup_selector, 
				sprintf('justify-content: %1$s;align-items: %2$s;', 
					$this->get_flex_alignment('justify', 'row', $location_x, 'center'),
					$this->get_flex_alignment('align', 'column', $location_y, 'center')
				),
				substr($device, 0, 1) 
			);
			if ( !( $module->props['popup_close'] === 'on' && $close_click[$device] === 'on') ) 
				$this->set_style( $module->slug, $popup_selector, sprintf('cursor: %s;', DDB_CLOSE_CURSOR), substr($device, 0, 1) );
		}

		$module->generate_styles(
			array(
				'base_attr_name' => 'overlay_color',
				'selector'       => $popup_selector,
				'css_property'   => 'background-color',
				'render_slug'    => $module->slug,
			)
		);

		$module->generate_styles(
			array(
				'base_attr_name' => 'popup_max_width',
				'selector'       => $popup_selector . ' .ddb-popup',
				'css_property'   => 'max-width',
				'render_slug'    => $module->slug,
			)
		);
		$module->generate_styles(
			array(
				'base_attr_name' => 'popup_max_height',
				'selector'       => $popup_selector . ' .ddb-popup',
				'css_property'   => 'max-height',
				'render_slug'    => $module->slug,
			)
		);	
		$module->generate_styles(
			array(
				'base_attr_name' 	=> 'content_background_color',
				'selector'       	=> $popup_selector . ' .ddb-popup > div',
				'css_property'   	=> 'background-color',
				'render_slug'    	=> $module->slug,
			)
		);

		if ( $module->props['popup_close'] === 'on' ) {
			$popup_close = $this->render_icon( array( 
				'module' => $module, 
				'icon_attr' => 'popup_close_font_icon', 
				'color_attr' => 'popup_close_text_color', 
				'size_attr' => 'popup_close_font_size', 
				'selector' =>  $popup_selector . ' .ddb-popup-icon',
				'attrs' => array( 'class' => 'ddb-popup-icon ddb-btn ddb-abs ddb-tr ddb-p-m')
			) ); 
		} 

		if ( $args['layout_popup'] ) {
			$popup_content = self::render_divi_layout( array('internal' => false, 'layout_id' => $module->props['divi_layout']) );
			
		} else {
			$popup_content = sprintf('<div class="ddb-custom-%2$s ddb-f ddb-p-xl ddb-bss">%1$s</div>', 
				$this->render_custom_content( $module, array( 'selector' => $popup_selector, 'slug' => $slug ) ), $slug 
			);			
		}
	
		if ( $module->props['popup_hover'] === 'on' ) $attrs['class'] .= ' ddb-show-hover';
		if ( $module->props['popup_disable_page_scroll'] === 'on' ) $attrs['class'] .= ' ddb-disable-page-scroll';
		if ( isset( $args['item_popup'] ) ) $attrs['class'] .= ' ddb-item-popup';
		$this->set_effect_attrs( $attrs, $module->props );
		
		return $multi_view->render_element(
			array(
				'tag'       => 'div',
				'content'	=> sprintf( '<div class="ddb-popup ddb-rel %3$s">%1$s%2$s</div>', $popup_close, $popup_content, $popup_id . '_inner' ),
				'attrs'		=> $attrs,
				'classes'   => array( 'ddb-c-btn' => array( 'popup_close' => 'on', 'popup_close_click' => 'on' ) )
			)
		);
	}

	public function render_text( $icon_args ) {
		$defaults = array(
			'attrs' => array(),
			'custom' => false,
		);
		$args = wp_parse_args( $icon_args, $defaults );
		
		$module = $args['module'];
		$attrs = $args['attrs'];

		$text_output = '';
		$text_values = 	et_pb_responsive_options()-> get_property_values( $module->props, $args['text_attr'], $module->props[$args['text_attr']], true );
		if ( et_pb_hover_options()->is_enabled( $args['text_attr'], $module->props ) ) 	
			$text_values['hover'] = et_pb_hover_options()->get_value( $args['text_attr'], $module->props, $text_values['desktop']);
		
		if ( $args['custom'] ) {
			$multi_value = false;
			$multi_attrs = array();  
			foreach ( $text_values as $mode => $value ) {
				if ( $text_values['desktop'] !== $value || ( isset($text_values['hover'] ) && $text_values['hover'] !== $value ) ) $multi_value = true;
				$multi_attrs["data-{$mode}"] = strip_tags( html_entity_decode( $value ), '<br>' );
			}
			if ( $multi_value ) {
				$attrs['class'] .= ' ddb-multi-value ddb-hide-tablet ddb-hide-phone';
				$attrs = array_merge( $attrs, $multi_attrs );
			}
			$attrs_output  = '';
			foreach( $attrs as $attr => $value){
				$attrs_output .= ' '. $attr . '="' . $value . '"';
			}
			$text_output = sprintf('<span%2$s>%1$s</span>', $multi_attrs['data-desktop'], $attrs_output );
			
		} else {
			$multi_view = et_pb_multi_view_options( $module );
			$text_output = $multi_view->render_element(
				array(
					'content' => '{{' . $args['text_attr'] .'}}',
					'attrs'   => $attrs,
				)
			);
		}
		return $text_output;
	}

	public function render_title($module, $attr, $heading_attr, $heading_level, $title_classes, $url = false, $prefix = '') {
		$multi_view	= et_pb_multi_view_options( $module );
		$title_attrs = array();
		$title_tag   = $url && '' !== $module->props["{$prefix}cc_url"] ? 'a' : 'span';
		if ( 'a' === $title_tag ) {
			$title_attrs['href'] = $module->props["{$prefix}cc_url"];

			if ( 'on' === "{$prefix}cc_url_new_window" ) {
				$title_attrs['target'] = '_blank';
			}
		}
		$title = $multi_view->render_element(
			array(
				'tag'     => $title_tag,
				'content' => '{{' . $attr . '}}',
				'attrs'   => $title_attrs,
			)
		);
		if ( '' !== $title ) {
			$title = sprintf(
				'<%1$s class="%3$s">%2$s</%1$s>',
				et_pb_process_header_level( $module->props[$heading_attr . '_level'], $heading_level ),
				et_core_esc_previously( $title ),
				esc_attr($title_classes)
			);
		}
		return $title;
	}

	public function set_margin_alignment($hali) { 
		return $hali === 'right' ? 'margin-left:auto;' : ($hali === 'left' ? 'margin-right:auto;' : 'margin-left:auto;margin-right:auto;'); 
	}

	public function set_style($render_slug, $selector, $declaration, $device = 'all') {
		$style = array( 'selector' => $selector, 'declaration' => $declaration);
		if ( $device === 'd' )
			$style['media_query'] = ET_Builder_Element::get_media_query( 'min_width_981' );
		else if ( $device === 't' )
			$style['media_query'] = ET_Builder_Element::get_media_query( '768_980' );
		else if ( $device === 'p' )
			$style['media_query'] = ET_Builder_Element::get_media_query( 'max_width_767' );
		ET_Builder_Element::set_style( $render_slug, $style);
	}

	public function set_effect_attrs( &$attrs, $props, $args = array()) {
		$defaults = array(
			'lazy_loading' => false,
			'prefix' => '',
		);
		$args = wp_parse_args( $args, $defaults );
		$prefix = $args['prefix'] !== '' ? $args['prefix'] . '_' : ''; 
		
		$effect_in = $props["{$prefix}effect_in"] !== 'none';
		$effect_out = '';
		if ( $effect_in ) { 
			$attrs['data-effect-in'] = esc_attr( $props["{$prefix}effect_in"] );
			$attrs['data-intensity-in'] = esc_attr( trim( $props["{$prefix}effect_in_intensity"], '%' ) );
			if ($args['lazy_loading']) 
				isset($attrs['class']) ? $attrs['class'] .= ' ddb-waypoint' : $attrs['class'] = 'ddb-waypoint';	
		}
		
		if ( !$args['lazy_loading'] && isset( $props["{$prefix}effect_out"] ) ) {
			$effect_out = $props["{$prefix}effect_out"] !== 'none';
		
			if ( $effect_out ) {
				$attrs['data-effect-out'] 	= esc_attr( $props["{$prefix}effect_out"] );
				$attrs['data-intensity-out'] = esc_attr( trim($props["{$prefix}effect_out_intensity"], '%') );
			}
		}
		if ( $effect_in || $effect_out ) {
			if ( $props["{$prefix}effect_opacity"] === 'on' ) $attrs['data-effect-opacity'] = esc_attr( trim($props["{$prefix}effect_opacity_intensity"], '%' ) );
			$attrs['data-effect-ms'] = esc_attr( trim( $props["{$prefix}effect_ms"], 'ms' ) ); 
		}	
	}

}