<?php

class DDBUILDER_Gallery_Module extends ET_Builder_Module {

	public $slug       = 'dondivi_gallery';
	public $vb_support = 'on';
	public $icon_path;

	protected $module_credits = array(
		'module_uri' => 'https://dondivi.com/gallery',
		'author'     => 'DonDivi',
		'author_uri' => 'https://dondivi.com/',
	);

	public function init() {
		$this->name 		= esc_html__( 'DonDivi Gallery', 'dd_builder' );
		$this->icon_path  	= plugin_dir_path( DDB_PLUGIN_PATH ) . 'builder/icon.svg';

		$this->main_css_element	= '%%order_class%%';
		
		$this->settings_modal_toggles = array(
			'general'    => array(
				'toggles' => array(
					'main_content' 	=> esc_html__( 'Gallery', 'et_builder' ),
					'images' 		=> esc_html__( 'Images', 'et_builder' ),
				),
			),
			'advanced'    => array(
				'toggles' => array(
					'main_content'  => esc_html__( 'Gallery', 'et_builder' ),
					'images' 		=> esc_html__( 'Images', 'et_builder' ),
					'overlay' 		=> esc_html__( 'Overlay', 'et_builder' ),
					'metadata' 		=> array(
                        'title' => sprintf( '%1$s %2$s', esc_html__( 'Overlay', 'et_builder' ), et_builder_i18n( 'Text' ) ),
                        'tabbed_subtoggles' => true,
                        'sub_toggles' => array(
                            'title' => array(
                                'name' => et_builder_i18n( 'Title' ),
							),
                            'caption' => array(
                                'name' => esc_html__( 'Caption', 'et_builder' ),
							),
						),
                    ),
					'lightbox' 	=> esc_html__( 'Lightbox', 'et_builder' ),
					'featured' 	=> esc_html__( 'Featured Image', 'et_builder' ),
					'buttons' 	=> array(
						'sub_toggles'       => array(
							'filters'     => array(
								'name' => esc_html__( 'Filters', 'et_builder' ),
							),
							'pagination' => array(
								'name' => esc_html__( 'Pagination', 'et_builder' )
							),
						),
						'tabbed_subtoggles' => true,
						'title'             => esc_html__( 'Navigation', 'et_builder' )
					),
					'button'	=> esc_html__( 'Load More', 'dd_builder' ),
					'effects' 	=> esc_html__( 'Image Animation', 'et_builder' ),
				),
			),
		);

		$this->advanced_fields = array(
			'borders' => array(
				'default' => array(),
				'image' => array (
					'label_prefix'   => et_builder_i18n( 'Image' ),
					'css' => array(
						'main' => array(
							'border_radii' =>  "{$this->main_css_element} .ddb-image-wrapper",
							'border_styles' => "{$this->main_css_element} .ddb-image-wrapper",
						),													
					),	
					'toggle_slug' 	=> 'images',				
				),
				'filter' => array (
					'label_prefix'    => esc_html__( 'Filter', 'et_builder' ),
					'css' => array(
						'main' => array(
							'border_radii' =>  "{$this->main_css_element} .ddb-filter",
							'border_styles' => "{$this->main_css_element} .ddb-filter",
						),													
					),	
					'depends_on'    	=> array( 'filterable' ),
					'depends_show_if' 	=> 'on',
					'toggle_slug' 		=> 'buttons',
					'sub_toggle'		=> 'filters',				
				),
				'page' => array (
					'label_prefix'    => esc_html__( 'Pagination', 'et_builder' ),
					'css' => array(
						'main' => array(
							'border_radii' =>  "{$this->main_css_element} .ddb-gallery-page",
							'border_styles' => "{$this->main_css_element} .ddb-gallery-page",
						),													
					),	
					'depends_on'        => array( 'show_pagination' ),
					'depends_show_if' 	=> 'on',
					'toggle_slug' 		=> 'buttons',
					'sub_toggle'		=> 'pagination',	
				),
			),
			'box_shadow' => array(
				'default' => array(),
				'image' => array (
					'label'	=>	et_builder_i18n( 'Image' ),
					'css' 	=> array(
						'main' => "{$this->main_css_element} .ddb-image-wrapper" 														
					),					
				),
			),
			'button'        => array(
				'button' => array(
					'label'          => et_builder_i18n( 'Button' ),
					'css'            => array(
						'main'      => "{$this->main_css_element} .ddb-load-more-button",
						'alignment' => "{$this->main_css_element} .ddb-load-more",
					),
					'no_rel_attr'    => true,
					'use_alignment'  => true,
					'toggle_slug' 	=> 'button', 
				),
			),
			'fonts' => array(
				'title'      => array(
					'label'        => et_builder_i18n( 'Title' ),
					'css'          => array(
						'main'  => "{$this->main_css_element} .ddb-overlay-title",
					),
					'header_level' => array(
						'default' => 'h3',
					),
					'hide_text_shadow'  => true,
					'depends_show_if' 	=> 'on',
					'toggle_slug' 		=> 'metadata',
					'sub_toggle'		=> 'title',
				),
				'caption'    => array(
					'label'           => esc_html__( 'Caption', 'et_builder' ),
					'css'             => array(
						'main'  		=> "{$this->main_css_element} .ddb-overlay-caption",
					),
					'line_height'     => array(
						'default'        => '1.7em',
						'range_settings' => array(
							'step' => '0.1',
						),
					),
					'hide_text_shadow'  => true,
					'depends_show_if' 	=> 'on',
					'toggle_slug' 		=> 'metadata',
					'sub_toggle'		=> 'caption',
				),
				'filter'   => array(
					'label'          	=> esc_html__( 'Filter', 'et_builder' ),
					'css'            	=> array(
						'main' => '%%order_class%% .ddb-filter',
						'hover' => '%%order_class%% .ddb-filter.active',
						'text_align'  => '%%order_class%% .ddb-gallery-filters',
					),
					'text_shadow'     	=> array( 'show_if'	=> array( 'filterable' => 'on' ) ), 
					'depends_show_if'	=> 'on',
					'toggle_slug' 		=> 'buttons',
					'sub_toggle'		=> 'filters'
					
				),
				'page'   => array(
					'label'          => esc_html__( 'Pagination', 'et_builder' ),
					'css'            => array(
						'main' 			=> "{$this->main_css_element} .ddb-gallery-page",
						'text_align'  	=> "{$this->main_css_element} .ddb-gallery-pagination",
					),
					'hide_text_shadow'  => true,
					'depends_show_if' 	=> 'on',
					'toggle_slug' 		=> 'buttons',
					'sub_toggle'		=> 'pagination'
				),
			),
			'image_icon'     => array(
				'image_icon' => array( 
					'label'           => esc_html__( 'Gallery', 'et_builder' ),
					'css'             => array(
						'main'    => '%%order_class%% .ddb-gallery',
					),
					'margin_padding'  => array(
						'css' => array(
							'main' => '%%order_class%% .ddb-gallery',
							'important' => 'all',
						),
					),
					'toggle_slug'     => 'main_content',
				),
				'overlay' => array( 
					'label'           => esc_html__( 'Overlay', 'et_builder' ),
					'css'             => array(
						'main' => '%%order_class%% .ddb-overlay', 
					),
					'margin_padding' 	=> array(
						'use_margin'	=> false,
					),
					'toggle_slug'     	=> 'overlay',
				),				
				'filter' => array( 
					'label'           => esc_html__( 'Filter', 'et_builder' ),
					'css'             => array(
						'main' => '%%order_class%% .ddb-filter', 
						'padding' => '%%order_class%% .ddb-filter'
					),
					'depends_show_if' 	=> 'on',
					'toggle_slug'     	=> 'buttons',
				),
				'page' => array( 
					'label'           => esc_html__( 'Pagination', 'et_builder' ),
					'css'             => array(
						'main' => '%%order_class%% .ddb-gallery-page', 
						'padding' => '%%order_class%% .ddb-gallery-page'
					),
					'depends_show_if' 	=> 'on',
					'toggle_slug'     	=> 'buttons',
				),
			),
			'text' => false,
		);

		$this->custom_css_fields = array(
			'image_css' => array(
				'label'    => esc_html__( 'Images', 'et_builder' ),
				'selector' => '%%order_class%% .ddb-gallery-item .ddb-image-wrapper'
			),
			'featured_image_css' => array(
				'label'    => esc_html__( 'Featured Image', 'et_builder' ),
				'selector' => '%%order_class%% .ddb-gallery-item.ddb-gallery-item--featured .ddb-image-wrapper'
			),
			'featured_image_before_css' => array(
				'label'    => sprintf( '%1$s %2$s', esc_html__( 'Featured Image', 'et_builder' ), et_builder_i18n( 'Before' ) ),
				'selector' => '%%order_class%% .ddb-gallery-item.ddb-gallery-item--featured .ddb-image-wrapper:before'
			),
		);
	}

	public function get_fields() {
		$image_sizes = array('full' => esc_html__( 'Full', 'et_builder' ) );
		foreach ( wp_get_registered_image_subsizes() as $key=>$val )
			$image_sizes [$key] = sprintf('%1$s (%2$s%3$s)%4$s', ucfirst($key), $val['width'], $val['height'] ? 'x' . $val['height'] : '', $val['crop'] ? ' Cropped' : '');

		$helper = new DonDiviBuilderFields();
		$fields = array(
			'gallery_ids' => array(
				'label'            	=> esc_html__( 'Images', 'et_builder' ),
				'description'      	=> esc_html__( 'Choose the images that you would like to appear in the image gallery.', 'et_builder' ),
				'type'        		=> 'upload-gallery',
				'computed_affects' => array(
					'__gallery',
				),
				'toggle_slug'      	=> 'main_content',
			),
			'image_count' 	=> array(
				'label'           => esc_html__( 'Image Count', 'et_builder' ),
				'type'            => 'text',
				'description'     => esc_html__( 'Define the number of images that should be displayed per page.', 'et_builder' ),
				'toggle_slug'     => 'main_content',
			),
			'gallery_orderby'        => array(
				'label'            => esc_html__( 'Image Order', 'et_builder' ),
				'description'      => esc_html__( 'Select an ordering method for the gallery. This controls which gallery items appear first in the list.', 'et_builder' ),
				'type'             => $this->is_loading_bb_data() ? 'hidden' : 'select',
				'options'          => array(
					'default'		=> et_builder_i18n( 'Default' ),
					'date_desc'  	=> esc_html__( 'Date: new to old', 'et_builder' ),
					'date_asc'   	=> esc_html__( 'Date: old to new', 'et_builder' ),
					'title_asc'  	=> esc_html__( 'Title: a-z', 'et_builder' ),
					'title_desc' 	=> esc_html__( 'Title: z-a', 'et_builder' ),
					'rand' 			=> esc_html__( 'Random', 'et_builder' ),
				),
				'default'          => 'default',
				'class'            => array( 'et-pb-gallery-ids-field' ),
				'toggle_slug'      => 'main_content',
				'computed_affects' => array(
					'__attachments',
				),
			),
			'filterable' 	=> array(
				'label'       	=> esc_html__( 'Filterable Gallery', 'dd_builder' ),
				'type'         	=> 'yes_no_button',
				'options'     	=> $helper->get_yes_no_button_options(),
				'affects' 		=> array( 
					'filter_font', 'filter_text_align', 'filter_text_color', 'filter_font_size', 'filter_letter_spacing', 'filter_line_height',
					'filter_custom_padding', 'filter_custom_margin', 
				),
				'default' 		=> 'off',
				'toggle_slug' 	=> 'main_content',
			),
			'load_more' 	=> array(
				'label'            	=> esc_html__( 'Load More', 'dd_builder' ),
				'type'             	=> 'yes_no_button',
				'options'          	=> $helper->get_yes_no_button_options(),
				'default' 			=> 'off',
				'toggle_slug'      	=> 'main_content',
			),
			'load_more_text' 	=> array(
				'label'           	=> esc_html__( 'Load More Text', 'dd_builder' ),
				'description'     	=> esc_html__( 'Input your desired button text, or leave blank for no button.', 'et_builder' ),
				'type'            	=> 'text',
				'show_if'			=> array( 'load_more' => 'on' ),
				'mobile_options'  	=> true,
				'hover'           	=> 'tabs',
				'dynamic_content' 	=> 'text',
				'toggle_slug'     	=> 'main_content',
			),
			'show_pagination'  	=> array(
				'label'            => esc_html__( 'Show Pagination', 'et_builder' ),
				'description'      => esc_html__( 'Enable or disable pagination for this feed.', 'et_builder' ),
				'type'             => 'yes_no_button',
				'options'          => $helper->get_yes_no_button_options(),
				'affects' 	=> array( 
					'page_font', 'page_text_align', 'page_text_color', 'page_font_size', 'page_letter_spacing', 'page_line_height', 
					'page_background_color', 'page_custom_margin', 'page_custom_padding', 'page_notice' 
				),
				'default' 			=> 'on',
				'mobile_options'   => true,
				'toggle_slug'      => 'main_content',
				
			),
			'show_pagination_tablet' => array(
				'type' 		=> 'skip',
				'affects' 	=> array( 
					'page_font', 'page_text_align', 'page_text_color', 'page_font_size', 'page_letter_spacing', 'page_line_height',
					'page_background_color', 'page_custom_margin', 'page_custom_padding' ),
			),
			'show_pagination_phone' => array(
				'type' 		=> 'skip',
				'affects' 	=> array( 
					'page_font', 'page_text_align', 'page_text_color', 'page_font_size', 'page_letter_spacing', 'page_line_height',
					'page_background_color', 'page_custom_margin', 'page_custom_padding' 
				),
			),
			'image_size' => array(
                'label'             => esc_html__( 'Image Size', 'dd_builder' ),
                'type'              => 'select',
                'options'           => $image_sizes,
                'default'           => 'full',
                'toggle_slug'       => 'images',
                'computed_affects' 	=> array(
					'__attachments',
				),
            ),
			'image_zoom' => array(
				'label'            => esc_html__( 'Enable Zoom Effect', 'dd_builder' ),
				'type'             => 'yes_no_button',
				'options'          => $helper->get_yes_no_button_options(),
				'default' 			=> 'off',
				'toggle_slug'    	=> 'images',
			),
			'overlay' => array(
				'label'            => esc_html__( 'Enable Overlay', 'dd_builder' ),
				'type'             => 'yes_no_button',
				'options'          =>  $helper->get_yes_no_button_options(),
				'affects' => array(
					'title_level', 'title_font', 'title_line_height', 'title_font_size', 'title_letter_spacing', 'title_text_color', 'title_text_align',
					'caption_font', 'caption_text_color', 'caption_line_height', 'caption_font_size', 'caption_letter_spacing','caption_text_align',
				),
				'default' 			=> 'off',
				'toggle_slug'    	=> 'images',
			),
			'overlay_title' => array(
				'label'            	=> esc_html__( 'Show Title', 'et_builder' ),
				'type'             	=> 'yes_no_button',
				'options'          	=> $helper->get_yes_no_button_options(),
				'show_if'			=> array( 'overlay' => 'on'),
				'default' 			=> 'off',
				'mobile_options'   	=> true,
				'toggle_slug'    	=> 'images',
			),
			'overlay_caption' => array(
				'label'            	=> esc_html__( 'Show Caption', 'dd_builder' ),
				'type'             	=> 'yes_no_button',
				'options'          	=> $helper->get_yes_no_button_options(),
				'default' 			=> 'off',
				'show_if'			=> array( 'overlay' => 'on'),
				'toggle_slug'    	=> 'images',
			),
			'on_click'   =>  array(
				'label'         	=> esc_html__( 'On Click', 'dd_builder' ),
				'type'            	=> 'select',
                'options'          	=>  array(
                    'none' 		=>  esc_html__( 'None', 'et_builder' ),
					'lightbox' 	=>  esc_html__( 'Open in Lightbox', 'et_builder' ),
                    'url' 		=>  esc_html__( 'Open URL', 'dd_builder' ),
                ),
				'default' 			=> 'none',
				'toggle_slug'     	=> 'images',
            ),
			'lightbox_title' => array(
				'label'            => esc_html__( 'Show Title in Lightbox', 'dd_builder' ),
				'type'             => 'yes_no_button',
				'options'          => $helper->get_yes_no_button_options(),
				'default' => 'on',
				'show_if'  => array( 'on_click' => 'lightbox' ),
				'toggle_slug'      => 'images',
				'mobile_options'   => true,
				'hover'            => 'tabs',
			),
			'lightbox_caption' => array(
				'label'            	=> esc_html__( 'Show Caption in Lightbox', 'dd_builder' ),
				'type'             	=> 'yes_no_button',
				'options'          	=> $helper->get_yes_no_button_options(),
				'default' 			=> 'on',
				'show_if'  => array( 'on_click' => 'lightbox' ),
				'toggle_slug'      => 'images',
				'mobile_options'   => true,
				'hover'            => 'tabs',
			),
			'layout'              => array(
				'label'            	=> et_builder_i18n( 'Layout' ),
				'description'      	=> esc_html__( 'Toggle between the various gallery layout types.', 'et_builder' ),
				'type'             	=> 'select',
				'options'          	=> array(
					'grid' 		=> esc_html__( 'Grid', 'et_builder' ),
					'masonry' 	=> esc_html__( 'Masonry', 'dd_builder' ),
				),
				'default' 			=> 'masonry',
				'tab_slug'         	=> 'advanced',
				'toggle_slug'      	=> 'main_content',
			),
			'layout_horizontal'  => array(
				'label'            	=> esc_html__( 'Horizontal Order', 'dd_builder' ),
				'type'             	=> 'yes_no_button',
				'options'          	=> $helper->get_yes_no_button_options(),
				'default' 			=> 'off',
				'show_if' 			=> array('layout' => 'masonry'),
				'tab_slug'     		=> 'advanced',
				'toggle_slug'      	=> 'main_content',
			),
			'layout_columns'     => array(
				'label'           	=> esc_html__( 'Columns', 'et_builder' ),
				'default'         	=> '3',
				'range_settings'  	=> array(
					'min_limit' => 0,
					'min'  => '1',
					'max'  => '10',
					'step'  => '1',
				),
				'type'            	=> 'range',
				'unitless'			=> true,
				'mobile_options'  	=> true,
				'tab_slug'         	=> 'advanced',
				'toggle_slug'     	=> 'main_content',
			),
			'layout_gap'     => array(
				'label'           	=> esc_html__( 'Gap', 'et_builder' ),
				'default'         	=> '10px',
				'type'            	=> 'range',
				'fixed_unit'		=> 'px',
				'mobile_options'  	=> true,
				'tab_slug'         	=> 'advanced',
				'toggle_slug'      	=> 'main_content',
			),
			'image_alignment' => array(
				'label'             => esc_html__( 'Image Alignment', 'et_builder' ),
				'type'            	=> 'text_align',
				'options'         	=> et_builder_get_text_orientation_options( array( 'justified' )),
				'options_icon'    	=> 'module_align',
				'default'			=> 'left',
				'mobile_options'	=> true,
				'tab_slug'        	=> 'advanced',
				'toggle_slug'     	=> 'images',
			),
			'fullwidth'              => array(
				'label'            	=> esc_html__( 'Fit To Container', 'dd_builder' ),
				'description'      	=> esc_html__( 'Whether or not to show the images will take up all their container width.', 'dd_builder' ),
				'type'             => 'yes_no_button',
				'options'          => $helper->get_yes_no_button_options(),
				'default' 			=> 'on',
				'mobile_options'	=> true,
				'tab_slug'        	=> 'advanced',
				'toggle_slug'    	=> 'images',
			),
			'featured_span'        => array(
				'label'          	 => esc_html__( 'Column Span', 'dd_builder' ),
				'type'            	=> 'range',
				'default'         	=> '1',
				'unitless'    		=> true,
				'mobile_options'  	=> true,
				'sticky'          	=> true,
				'tab_slug'        	=> 'advanced',
				'toggle_slug'     	=> 'featured',
			),
			'featured_width'        => array(
				'label'           => esc_html__( 'Width', 'et_builder' ),
				'description'     => esc_html__( 'Adjust the width of the featured image.', 'et_builder' ),
				'type'            => 'range',
				'allowed_values'  => et_builder_get_acceptable_css_string_values( 'width' ),
				'default'         => 'auto',
				'default_unit'    => 'px',
				'show_if_not'     => array( 'layout' => 'masonry' ),
				'mobile_options'  	=> true,
				'sticky'          => true,
				'tab_slug'        	=> 'advanced',
				'toggle_slug'     	=> 'featured',
			),
			'hover_overlay_color'    => array(
				'label'           	=> esc_html__( 'Overlay Background Color', 'et_builder' ),
				'description'     	=> esc_html__( 'Here you can define a custom color for the overlay', 'et_builder' ),
				'type'            	=> 'color-alpha',
				'custom_color'    	=> true,
				'default'			=> 'rgba(0, 0, 0, 0.7)',
				'show_if' 			=> array('overlay' => 'on'),
				'tab_slug'        	=> 'advanced',
				'toggle_slug'     	=> 'overlay',
				'mobile_options'  	=> true,
				'sticky'          	=> true,
			),
			'hover_icon'             => array(
				'label'           	=> esc_html__( 'Overlay Icon', 'et_builder' ),
				'description'     	=> esc_html__( 'Here you can define a custom icon for the overlay', 'et_builder' ),
				'type'            	=> 'select_icon',
				'class'           	=> array( 'et-pb-font-icon' ),
				'show_if' 			=> array('overlay' => 'on'),
				'tab_slug'        	=> 'advanced',
				'toggle_slug'     	=> 'overlay',
				'mobile_options'  	=> true,
				'sticky'          	=> true,
			),
			'hover_icon_color'        => array(
				'label'           	=> esc_html__( 'Overlay Icon Color', 'et_builder' ),
				'type'            	=> 'color-alpha',
				'custom_color'    	=> true,
				'show_if' 			=> array('overlay' => 'on'),
				'tab_slug'        	=> 'advanced',
				'toggle_slug'     	=> 'overlay',
				'mobile_options'  	=> true,
				'sticky'          	=> true,
			),
			'hover_icon_width' => array(
				'label'           	=> sprintf( '%1$s %2$s', esc_html__( 'Overlay Icon', 'et_builder' ), esc_html__( 'Width', 'et_builder' ) ),
				'type'             	=> 'range',
				'default_unit'		=> 'px',
				'default'			=> '30px',
				'show_if' 			=> array('overlay' => 'on'),
				'mobile_options'   	=> true,
				'sticky'           	=> true,
				'tab_slug'        	=> 'advanced',
				'toggle_slug'     	=> 'overlay',
			),
			'lightbox_background_color'    => array(
				'label'           	=> esc_html__( 'Background Color', 'et_builder' ),
				'type'            	=> 'color-alpha',
				'custom_color'    	=> true,
				'show_if' 			=> array( 'on_click' => 'lightbox' ),
				'mobile_options'  	=> true,
				'tab_slug'        	=> 'advanced',
				'toggle_slug'     	=> 'lightbox',			
			),
			'lightbox_opacity'     => array(
				'label'           	=> esc_html__( 'Fade Animation Opacity', 'dd_builder' ),
				'default'         	=> '0.8',
				'type'            	=> 'range',
				'unitless'         	=> true,
				'range_settings'  	=> array(
					'min'  => '0.5',
					'max'  => '1',
					'step'  => '0.1',
				),
				'show_if' 			=> array( 'on_click' => 'lightbox' ),
				'mobile_options'  	=> true,
				'tab_slug'         	=> 'advanced',
				'toggle_slug'      	=> 'lightbox',
			),
			'filter_background_color'    => array(
				'label'           	=> esc_html__( 'Background Color', 'et_builder' ),
				'type'            	=> 'color-alpha',
				'custom_color'    	=> true,
				'show_if' 			=> array( 'filterable' => 'on' ),
				'tab_slug'        	=> 'advanced',
				'toggle_slug'     	=> 'buttons',
				'sub_toggle'     	=> 'filters',
				'hover'  			=> 'tabs',
				'mobile_options'  	=> true,
				'sticky'          	=> true,
			),
			'page_notice' 			=> array(
				'type'            	=> 'warning',
				'value'           	=> true,
				'display_if'      	=> true,
				'message'         	=> esc_html__( 'The pagination is disabled for this view.', 'dd_builder' ),
				'depends_show_if' 	=> 'off',
				'tab_slug'         	=> 'advanced',
				'toggle_slug'     	=> 'buttons',
				'sub_toggle'     	=> 'pagination',
			),
			'page_background_color'    => array(
				'label'           	=> esc_html__( 'Background Color', 'et_builder' ),
				'type'            	=> 'color-alpha',
				'custom_color'    	=> true,
				'depends_show_if' 	=> 'on',
				'tab_slug'        	=> 'advanced',
				'toggle_slug'     	=> 'buttons',
				'sub_toggle'     	=> 'pagination',
				'hover'  			=> 'tabs',
				'mobile_options'  	=> true,
				'sticky'          	=> true,
			),  
			'__attachments' => array(
				'type'                => 'computed',
				'computed_callback'   => array( 'DDBUILDER_Gallery_Module', 'get_gallery' ),
				'computed_depends_on' => array(
					'gallery_ids',
					'image_size',
					'gallery_orderby',
				),
				'computed_minimum' => array(
					'gallery_ids',  
				),
			),
		);
		return array_merge( $fields, $helper->get_effect_fields( 'effects', false, 'zoom', '', '400ms', true ) );
	}

	public static function get_gallery( $args = array(), $conditional_tags = array(), $current_page = array() ) {
		$all_categories = array();
		$attachments = array();

		$defaults = array(
			'gallery_ids'      	=> array(),
			'gallery_orderby'  	=> '',
			'image_size' 		=> 'full',
		);

		$args = wp_parse_args( $args, $defaults );

		$query_args = array(
			'include'        => $args['gallery_ids'],
			'post_status'    => 'inherit',
			'post_type'      => 'attachment',
			'post_mime_type' => 'image',
		);

		if ( isset( $args['attachment_id'] ) ) { // from the Divi Gallery // maybe to remove this
			$query_args['attachment_id'] = $args['attachment_id'];
		}

		$query_args['orderby'] = 'post__in';

		switch ( $args['gallery_orderby'] ) {
			case 'date_desc':
				$query_args['orderby'] = 'date';
				$query_args['order']   = 'DESC';
				break;
			case 'date_asc':
				$query_args['orderby'] = 'date';
				$query_args['order']   = 'ASC';
				break;
			case 'title_asc':
				$query_args['orderby'] = 'title';
				$query_args['order']   = 'ASC';
				break;
			case 'title_desc':
				$query_args['orderby'] = 'title';
				$query_args['order']   = 'DESC';
				break;
			case 'rand':
				$query_args['orderby'] = 'rand';
				break;
		}

		$_attachments = get_posts( $query_args );
		
		foreach ( $_attachments as $key => $val ) {
			$attachments[ $key ]                  	= $_attachments[ $key ];
			$attachments[ $key ]->image_alt_text  	= get_post_meta( $val->ID, '_wp_attachment_image_alt', true );
			$attachments[ $key ]->category = array(); 
			$categories = get_the_terms( $val->ID, 'dondivi_media_category' );
			if ($categories) {
				foreach ($categories as $category) {
					if (!array_key_exists($category->term_id, $all_categories)) $all_categories['cat-' . $category->term_id]= $category->name;
					$attachments[ $key ]->category[]= 'cat-' . $category->term_id;
				}
			}
			$attachments[ $key ]->image_src  		= wp_get_attachment_image_src( $val->ID, $args['image_size'] );
			$attachments[ $key ]->image_src_full  	= wp_get_attachment_image_src( $val->ID, 'full' );
			$attachments[ $key ]->image_title 		= get_the_title( $val->ID );
			$attachments[ $key ]->image_caption     = wp_get_attachment_caption( $val->ID);
			$attachments[ $key ]->is_featured  	  	= (bool) get_post_meta( $val->ID, '_ddbuilder_featured', true );
			$attachments[ $key ]->destination_url  	= get_post_meta( $val->ID, '_ddbuilder_url', true );
		}		
		return array ( 'attachments' => $attachments, 'categories' => $all_categories);
	}

	private function _render_filters($categories) {
		$filters = '';
		if ($this->props['filterable'] === 'on') {
			$this->generate_styles(
				array(
					'hover'          => false,
					'base_attr_name' => 'filter_background_color',
					'selector'       => '%%order_class%% .ddb-filter',
					'hover'			 => true,
					'hover_selector' => '%%order_class%% .ddb-filter.active',
					'css_property'   => 'background-color',
					'render_slug'    => $this->slug,
				)
			);
			$filters = sprintf('<div class="ddb-filter ddb-btn ddb-gallery-btn ddb-inline-block ddb-p-m ddb-bss active" data-filter="all">%s</div>',
				esc_html__( 'All', 'et_builder' )
			);
			foreach ( $categories as $key => $val ) {
				$filters .= sprintf('<div class="ddb-filter ddb-btn ddb-gallery-btn ddb-inline-block ddb-p-m ddb-bss" data-filter="%2$s">%1$s</div>', $val, $key);
			}
			return sprintf('<div class="ddb-gallery-filters ddb-pb">%s</div>', $filters);
		}	
		return $filters;
	}

	public function multi_view_filter_value( $raw_value, $args ) {

		$name = isset( $args['name'] ) ? $args['name'] : '';

		if ( $raw_value && ('hover_icon' === $name ) ) {
			return et_pb_get_extended_font_icon_value( $raw_value, true );
		}
		return $raw_value;
	}

	public function render( $attrs, $content, $render_slug ) {

		if ($this->props['layout'] !== 'grid') {	
			wp_enqueue_script( 'ddb-imagesloaded' );
			wp_enqueue_script( 'ddb-masonry' );
		}
		wp_enqueue_script( 'ddb-gallery' );

		$helper 		= DonDiviBuilderHelper::get();
		$multi_view     = et_pb_multi_view_options( $this );
		$gallery_images = self::get_gallery(
			array(
				'gallery_ids' => $this->props['gallery_ids'],
				'gallery_orderby' =>$this->props['gallery_orderby'],
				'image_size' => $this->props['image_size'],
			)
		);

		$span = et_pb_responsive_options()->get_property_values($this->props, 'featured_span', $this->props['featured_span'], true );
		if ( $this->props['layout'] === 'masonry') {
			$columns 	= et_pb_responsive_options()->get_property_values($this->props, 'layout_columns', $this->props['layout_columns'], true );
			$gutter  	= et_pb_responsive_options()->get_property_values($this->props, 'layout_gap', $this->props['layout_gap'], true );

			foreach ( et_pb_responsive_options()->get_modes() as $device ) {
				$item_width = 100 / $columns[$device];
				$gutter_width =  intval( $gutter[$device] ) * ( $columns[$device]  - 1 ) / $columns[$device];
				$helper->set_style( 
					$render_slug, 
					'%%order_class%% .ddb-gallery-item, %%order_class%% .ddb-gallery-sizer', 
					sprintf( 'width: calc(%1$s%% - %2$spx);', $item_width, $gutter_width ),
					substr( $device, 0, 1 ) 
				);

				if ( intval($span[$device]) < intval($columns[$device]) ) {
					$item_width *= $span[$device];
					$gutter_width =  intval( $gutter[$device] ) * ( $columns[$device]  - $span[$device] ) / $columns[$device];
				} else {
					$item_width = 100;
					$gutter_width = 0;
				}
				$helper->set_style( 
					$render_slug, 
					'%%order_class%% .ddb-gallery-item.ddb-gallery-item--featured', 
					sprintf( 'width: calc(%1$s%% - %2$spx);', $item_width, $gutter_width ),
					substr( $device, 0, 1 ) 
				);
			}
			$this->generate_styles(
				array(
					'base_attr_name' => 'layout_gap',
					'selector'       => '%%order_class%% .ddb-gutter-sizer',
					'css_property'   => 'width',
					'render_slug'    => $render_slug,
				)
			);
			$this->generate_styles(
				array(
					'base_attr_name' => 'layout_gap',
					'selector'       => '%%order_class%% .ddb-gallery-item',
					'css_property'   => 'margin-bottom',
					'render_slug'    => $render_slug,
				)	
			);
		} else {
			$this->generate_styles(
				array(
					'base_attr_name' => 'layout_columns',
					'selector'       => '%%order_class%% .ddb-grid',
					'css_property'   => '--ddb-grid-columns',
					'render_slug'    => $render_slug,
				)	
			);
			$this->generate_styles(
				array(
					'base_attr_name' => 'layout_gap',
					'selector'       => '%%order_class%% .ddb-grid',
					'css_property'   => 'gap',
					'render_slug'    => $render_slug,
				)	
			);
			$this->generate_styles(
				array(
					'base_attr_name' => 'featured_width',
					'selector'       => '%%order_class%% .ddb-gallery-item.ddb-gallery-item--featured',
					'css_property'   => 'width',
					'render_slug'    => $render_slug,
				)	
			);
			foreach ( et_pb_responsive_options()->get_modes() as $device ) {
				$helper->set_style( 
					$render_slug, 
					'%%order_class%% .ddb-gallery-item.ddb-gallery-item--featured', 
					sprintf( 'grid-column-start: span %s;', $span[$device] ),
					substr( $device, 0, 1 ) 
				);
			}
		}

		$this->generate_styles(
			array(
				'base_attr_name' => 'image_alignment',
				'selector'       => '%%order_class%% .ddb-gallery-item',
				'css_property'   => 'text-align',
				'render_slug'    => $render_slug,
			)
		);

		$overlay_icon = '';
		if ($this->props['overlay'] === 'on') {
			$overlay_icon = $helper->render_icon( array( 
				'module' 	=> $this, 
				'icon_attr' => 'hover_icon', 
				'color_attr'=> 'hover_icon_color', 
				'size_attr' => 'hover_icon_width', 
				'selector' 	=>  '%%order_class%% .ddb-overlay-icon', 
				'attrs' 	=> array( 'class' => 'ddb-overlay-icon ddb-p-m' )
			));
			$this->generate_styles(
				array(
					'hover'          => false,
					'base_attr_name' => 'hover_overlay_color',
					'selector'       => '%%order_class%% .ddb-overlay',
					'css_property'   => array( 'background-color', 'border-color' ),
					'render_slug'    => $render_slug,
					'type'           => 'color',
				)
			);
		}

		$images = $this->props['layout'] !== 'grid' ? '<div class="ddb-gallery-sizer"></div><div class="ddb-gutter-sizer"></div>' : '';
		foreach ( $gallery_images['attachments'] as $id => $attachment ) {
			$overlay = '';
			if ($this->props['overlay'] === 'on') {
				$overlay_title = $overlay_caption = '';
				if ($attachment->image_title) {
					if ( $multi_view->has_value( 'overlay_title', 'on' ) ) {
						$overlay_title = $multi_view->render_element(
							array(
								'tag'        => et_pb_process_header_level( $this->props['title_level'], 'h3' ),
								'content'    => wptexturize( $attachment->image_title ),
								'attrs'      => array(
									'class' => 'ddb-overlay-title ddb-light',
								),
								'visibility' => array(
									'overlay_title' => 'on',
								),
							)
						);
					}
				}
					
				if ($attachment->image_caption) {
					if ( $multi_view->has_value( 'overlay_caption', 'on' ) ) {
						$overlay_caption = $multi_view->render_element(
							array(
								'tag'        => 'p',
								'content'    => wptexturize( $attachment->image_caption ),
								'attrs'      => array(
									'class' => 'ddb-overlay-caption',
								),
								'visibility' => array(
									'overlay_caption' => 'on',
								),
							)
						);
					}
				}
				$overlay = sprintf('<div class="ddb-overlay ddb-abs ddb-100 ddb-tl ddb-f ddb-f-cc ddb-column ddb-light"><div class="ddb-overlay-content">%1$s%2$s</div>%3$s</div>', 
					$overlay_title, 
					$overlay_caption,
					$overlay_icon
				);
			}

			$image_classes = array('ddb-gallery-item');
			if ($attachment->is_featured)
				$image_classes[]= 'ddb-gallery-item--featured';

			foreach ($attachment->category as $category) $image_classes[]=  $category;
			
			$image_wrapper = $multi_view->render_element(
				array(
					'tag'        	=> 'div',
					'content'		=> $helper->render_image( array( 
						'module' 	=> $this, 
						'img_src' 	=> $attachment->image_src[0], 
						'alt_value' => $attachment->image_alt_text 
					) ) . $overlay,
					'attrs'      => array(
						'class' => 'ddb-image-wrapper ddb-rel ddb-bss ddb-ovh', 
					),
					'classes'=> array( 'ddb-block' => array( 'fullwidth' => 'on'))
				)
			);

			if ($this->props['on_click'] !== 'none') { 
				$image_attrs = array( sprintf('href ="%s"', $this->props['on_click'] === 'url' ? $attachment->destination_url : $attachment->image_src_full[0]) ); 
				if ( $this->props['on_click'] === 'lightbox' && $this->props['lightbox_title'] === 'on' )
					$image_attrs[]= $attachment->image_title ? sprintf( 'data-title="%1$s"', $attachment->image_title ) : '';
				if ( $this->props['on_click'] === 'lightbox' && $this->props['lightbox_caption'] === 'on' )
					$image_attrs[]= $attachment->image_caption ? sprintf( 'data-caption="%s"', $attachment->image_caption ) : '';
				
				$href = $this->props['on_click'] === 'url' ? $attachment->destination_url : $attachment->image_src_full;
				$images .= sprintf( '<a class="%2$s"%3$s>%1$s</a>', $image_wrapper, implode(' ', $image_classes), implode(' ', $image_attrs));
				
				add_filter( 'et_late_global_assets_list', function( $assets, $assets_args, $et_dynamic_assets ) { // closures were added in PHP 5.3		
					if ( isset( $assets['et_jquery_magnific_popup'] ) ) 
						return $assets;	
					
					$assets_prefix = et_get_dynamic_assets_path();
					$assets['et_jquery_magnific_popup'] = array( 'css' => "{$assets_prefix}/css/magnific_popup.css" );
					return $assets;
				}, 10, 3 );
			} else  
              	$images .= sprintf( '<div class="%2$s">%1$s</div>', $image_wrapper, implode(' ', $image_classes));
		}

		$pagination = '';
		if ( $multi_view->has_value( 'show_pagination', 'on' ) ) {
			$this->generate_styles(
				array(
					'hover'          => false,
					'base_attr_name' => 'page_background_color',
					'selector'       => '%%order_class%% .ddb-gallery-page',
					'hover_selector' => '%%order_class%% .ddb-gallery-page.active',
					'hover' 		 => true,
					'css_property'   => 'background-color',
					'render_slug'    => $render_slug,
					'type'           => 'color',
				)
			);
			$pagination_classes = array( 'ddb-gallery-pagination ddb-mt' );
			$pagination = et_pb_responsive_options()->get_property_values($this->props, 'show_pagination', $this->props['show_pagination'], true );
			$responsive = et_pb_responsive_options()-> is_responsive_enabled($this->props, 'show_pagination');
			$pagination_classes[] = $helper->get_responsive_classnames($pagination, $responsive, 'ddb-pag', array('on'));
			$pagination = $multi_view->render_element(
				array(
					'tag'        => 'div',
					'attrs'      => array(
						//'classes' => array( 'ddb-pagination' => array( 'show_pagination' => 'on' ) ), // our JS code is being executed before Divi changes this class
						'class' => implode( ' ', $pagination_classes ), // so we need to use our responsive classes (ddb-pag(d-t-p) that become ddb-pagination) for now
					),
					'visibility' => array(
						'show_pagination' => 'on',
					),
				)
			);
		} // end foreach
	
		$load_more_button = '';
		if ( $this->props['load_more'] === 'on' ) {
			$load_more_classes = array( 'ddb-load-more ddb-mt' );
			$load_more_values = et_pb_responsive_options()->get_property_values( $this->props, 'load_more', $this->props['load_more'], true );
			$responsive = et_pb_responsive_options()-> is_responsive_enabled( $this->props, 'load_more' );
			$load_more_classes[] = $helper->get_responsive_classnames( $load_more_values, $responsive, 'ddb-l', array('on') );
			$load_more_text = $this->_esc_attr( 'load_more_text', 'limited' );		
			$load_more_button = $helper->get_custom_button( $this, 'button', 'load_more_text', $load_more_text, 'ddb-load-more-button', implode(' ', $load_more_classes ) );
		}
		
		$gallery_attrs = array( 'class' => "ddb-gallery ddb-{$this->props['layout']}" );
		if ( 0 !== intval( $this->props['image_count'] ) ) $gallery_attrs['data-per-page']= intval( $this->props['image_count'] );
		if ( $this->props['load_more'] === 'on' ) $gallery_attrs['data-load'] = '0';
		$helper->set_effect_attrs( $gallery_attrs, $this->props ); 

		if ($this->props['on_click'] === 'lightbox')  {
			$this->generate_styles(
				array(
					'hover'          => false,
					'base_attr_name' => 'lightbox_background_color',
					'selector'       => '.mfp-bg.ddb-mfp-bg',
					'css_property'   => 'background-color',
					'render_slug'    => $render_slug,
					'type'           => 'color',
				)
			);
			$this->generate_styles(
				array(
					'hover'          => false,
					'base_attr_name' => 'lightbox_opacity',
					'selector'       => '.mfp-fade.mfp-bg.ddb-mfp-bg.mfp-ready',
					'css_property'   => 'opacity',
					'render_slug'    => $render_slug,
				)
			);
			$gallery_attrs['class'].= ' ddb-lightbox';
		}
		if ($this->props['layout'] === 'masonry' && $this->props['layout_horizontal'] === 'on') {
			$gallery_attrs['class'].= ' ddb-horizontal';
		}
		if ($this->props['image_zoom'] === 'on') {
			$gallery_attrs['class'].= ' ddb-zoom';
		}
		
		$gallery = $multi_view->render_element(
			array(
				'tag'      	=> 'div',
				'content'	=> $images,
				'attrs'     => $gallery_attrs,
			)
		);

		return $this->_render_filters($gallery_images['categories']) . $gallery . $load_more_button . $pagination;
	}
}

new DDBUILDER_Gallery_Module;