<?php

class DDBUILDER_Hotspot_Module extends ET_Builder_Module {

    public $slug       = 'dondivi_hotspots';
    public $vb_support = 'on';
    public $icon_path;

    protected $module_credits = array(
        'module_uri' => 'https://dondivi.com/hotspots',
        'author'     => 'DonDivi',
        'author_uri' => 'https://dondivi.com/',
    );

    public function init() {
        $this->name 		    = esc_html__( 'DonDivi Hotspots', 'dd_builder' );
        $this->child_slug 	    = 'dondivi_hotspot';
        $this->child_item_text 	= esc_html__( 'Hotspot', 'dd_builder' );
        $this->icon_path        = plugin_dir_path( DDB_PLUGIN_PATH ) . 'builder/icon.svg';

        $this->settings_modal_toggles = array(
            'general'    => array(
                'toggles' => array(
                    'main_content'  => et_builder_i18n( 'Image' ),
                ),
            ),
        );

        $this->advanced_fields = array(
            'box_shadow' => array(
                'default' => array(
					'css'      => array(
						'main' => '%%order_class%% .ddb-hotspots-img',
					),
				),
            ),
            'borders'   => array(
				'default' => array(
					'css'      => array(
						'main' => array(
							'border_radii'  => '%%order_class%% .ddb-hotspots-img',
							'border_styles' => '%%order_class%% .ddb-hotspots-img',
						),
					),
				),
            ),
            'fonts'     => false,	
            'text'      => false,
        );

        $this->custom_css_fields = array(
            'hotspot_text' => array(
				'label'    => sprintf( '%1$s %2$s', esc_html__( 'Hotspots', 'dd_builder' ), et_builder_i18n( 'Text' ) ),
				'selector' => '%%order_class%% .ddb-hotspot-text',
			),
            'hotspot_image' => array(
				'label'    => sprintf( '%1$s %2$s', esc_html__( 'Hotspots', 'dd_builder' ), et_builder_i18n( 'Image' ) ),
				'selector' => '%%order_class%% .ddb-hotspot > img',
			),
            'hotspot_icon' => array(
				'label'    => sprintf( '%1$s %2$s', esc_html__( 'Hotspots', 'dd_builder' ), et_builder_i18n( 'Icon' ) ),
				'selector' => '%%order_class%% .ddb-hotspot > .ddb-icon',
			),
        );
    }

    public function get_fields() {
        return array(
            'src' => array(
                'label'             => et_builder_i18n( 'Image' ),
                'type'              => 'upload',
                'upload_button_text'=> esc_attr__( 'Upload an image', 'et_builder' ),
                'choose_text'       => esc_attr__( 'Choose an Image', 'et_builder' ),
                'update_text'       => esc_attr__( 'Set As Image', 'et_builder' ),
                'toggle_slug'       => 'main_content',
                'dynamic_content' 	=> 'image',		
            ),
            'alt'                         => array(
				'label'             => esc_html__( 'Image Alt Text', 'et_builder' ),
                'description'       => esc_html__( 'Define the HTML ALT text for your image here.', 'et_builder' ),
				'type'              => 'text',
				'tab_slug'          => 'custom_css',
				'toggle_slug'       => 'attributes',
				'dynamic_content'   => 'text',
			),
        );
    }
 
    public function render( $attrs, $content, $render_slug ) {
        wp_enqueue_script( 'ddb-hotspots' );

        $image = DonDiviBuilderHelper::get()->render_image( array( 
            'module' => $this, 
            'alt_attr' => 'alt',
            'img_attr' => 'src',
            'attrs' => array( 'class' => 'ddb-hotspots-img ddb-bss ddb-w-100'), 
        ) );
        return sprintf( '<div class="ddb-image-wrapper ddb-rel ddb-block">%1$s</div>%2$s%3$s', $image, '', $this->props['content'] );
    }
}

new DDBUILDER_Hotspot_Module;