<?php

class DDBUILDER_Popup_Module extends ET_Builder_Module {

	public $slug       = 'dondivi_popup';
	public $vb_support = 'on';
	public $icon_path;

	protected $module_credits = array(
		'module_uri' => 'https://dondivi.com/popup',
		'author'     => 'DonDivi',
		'author_uri' => 'https://dondivi.com/',
	);

	public function init() {
		$this->name 		= esc_html__( 'DonDivi Popup', 'dd_builder' );
		$this->icon_path   	= plugin_dir_path(DDB_PLUGIN_PATH) . 'builder/icon.svg';

		$this->main_css_element = '%%order_class%%';

		$this->settings_modal_toggles = array(
			'general'   => array(
				'toggles' => array(
					'main_content' 	=> esc_html__( 'Popup', 'dd_builder' ),
					'popup_content' => esc_html__( 'Content', 'et_builder' ),
				),
			),
			'advanced'   => array(
				'toggles' => array(
					'trigger'		=> esc_html__( 'Trigger Element', 'dd_builder' ),
					'popup' 		=> esc_html__( 'Popup', 'dd_builder' ),
					'popup_icon' 	=> sprintf( '%1$s %2$s', esc_html__( 'Popup', 'dd_builder' ), esc_html__( 'Image & Icon', 'et_builder' ) ),
					'title' 		=> sprintf( '%1$s %2$s', esc_html__( 'Popup', 'et_builder' ), et_builder_i18n( 'Title' )),
					'body'   		=> array(
						'title'             => sprintf( '%1$s %2$s', esc_html__( 'Popup', 'et_builder' ), et_builder_i18n( 'Body' ) ),
						'tabbed_subtoggles' => true,
						'bb_icons_support'  => true,
						'sub_toggles'       => array(
							'p'     => array(
								'name' => 'P',
								'icon' => 'text-left',
							),
							'a'     => array(
								'name' => 'A',
								'icon' => 'text-link',
							),
							'ul'    => array(
								'name' => 'UL',
								'icon' => 'list',
							),
							'ol'    => array(
								'name' => 'OL',
								'icon' => 'numbered-list',
							),
							'quote' => array(
								'name' => 'QUOTE',
								'icon' => 'text-quote',
							),
						),
					),
					'button' 	=> sprintf( '%1$s %2$s', esc_html__( 'Popup', 'dd_builder' ), et_builder_i18n( 'Button' ) ),
					'effects' 	=> sprintf( '%1$s %2$s', esc_html__( 'Popup', 'dd_builder' ), esc_html__( 'Animation', 'et_builder' ) ),
				),
			),
		);

		$this->advanced_fields = array(
			'background' 	=> false,
			'borders' 		=> array(
				'popup' => array (
					'label_prefix'    => esc_html__( 'Popup', 'dd_builder' ),
					'css' => array(
						'main' => array(
							'border_radii' =>  "{$this->main_css_element}_popup .ddb-custom-popup",
							'border_styles' => "{$this->main_css_element}_popup .ddb-custom-popup",
						),													
					),	 
					'depends_on'    	=> array( 'item_content' ),
					'depends_show_if' 	=> 'custom',
					'toggle_slug' 		=> 'popup',
				),
			),
			'box_shadow' 	=> array(
				'default' => array(
					'label' => esc_html__( 'Popup', 'dd_builder' ),
					'css'      => array(
						'main' => '%%order_class%%_popup .ddb-custom-popup',
					),
					'depends_on'    	=> array( 'item_content' ),
					'depends_show_if' 	=> 'custom',
				),
			),
			'button'      	=> array(
				'button' => array(
					'label'          => et_builder_i18n( 'Button' ),
					'css'            => array(
						'main'         => "{$this->main_css_element}_popup_inner .ddb-popup-button",
						'alignment'    => "{$this->main_css_element}_popup .et_pb_button_module_wrapper",
					),
					'use_alignment'  => true,
					'box_shadow'     => array(
						'css' => array(
							'main' => "{$this->main_css_element}_popup_inner .ddb-popup-button",
						),
					),
					'toggle_slug' 	=> 'button', 
				),
				'trigger' => array(
					'label'          => esc_html__( 'Trigger Button' , 'dd_builder'),
					'css'            => array(
						'main'         => "{$this->main_css_element} .et_pb_button.ddb-trigger-element",
					),
					'box_shadow'     => array(
						'css' => array(
							'main' => '%%order_class%% .ddb-trigger-element',
						),
					),
					'toggle_slug' 	=> 'trigger',	
				),
			),	
			'fonts'          => array(
				'title' => array(
					'label'        => et_builder_i18n( 'Title' ),
					'css'          => array(
						'main'  => "{$this->main_css_element}_popup .ddb-popup-title, {$this->main_css_element}_popup .ddb-popup-title a",
					),
					'header_level' => array(
						'default' => 'h4',
					),
					'text_shadow' 	=> array( 'show_if' => array( 'item_content' => 'custom' ) ),
					'toggle_slug' 	=> 'title',
				),
				'body'   => array(
					'label'         => et_builder_i18n( 'Body' ),
					'css'           => array(
						'main' => "{$this->main_css_element}_popup .ddb-popup-body",
					),
					'text_shadow' 	=> array( 'show_if' => array( 'item_content' => 'custom' ) ),
					'toggle_slug' 	=> 'body',
					'sub_toggle'   	=> 'p',
				),
				'body_link'     => array(
					'label'       	=> et_builder_i18n( 'Link' ),
					'css'         	=> array(
						'main'  => "{$this->main_css_element}_popup .ddb-popup-body a",
					),
					'text_shadow' 	=> array( 'show_if' => array( 'item_content' => 'custom' ) ),
					'toggle_slug' 	=> 'body',
					'sub_toggle'  	=> 'a',
				),
				'body_ul'       => array(
					'label'       	=> esc_html__( 'Unordered List', 'et_builder' ),
					'css'         	=> array(
						'main'        => "{$this->main_css_element}_popup .ddb-popup-body ul li",
						'item_indent' => "{$this->main_css_element}_popup .ddb-popup-body ul",
					),
					'text_shadow' 	=> array( 'show_if' => array( 'item_content' => 'custom' ) ),
					'toggle_slug' 	=> 'body',
					'sub_toggle'  	=> 'ul',
				),
				'body_ol'       => array(
					'label'       	=> esc_html__( 'Ordered List', 'et_builder' ),
					'css'         	=> array(
						'main'        => "{$this->main_css_element}_popup .ddb-popup-body ol li",
						'item_indent' => "{$this->main_css_element}_popup .ddb-popup-body ol",
					),
					'text_shadow' 	=> array( 'show_if' => array( 'item_content' => 'custom' ) ),
					'toggle_slug' 	=> 'body',
					'sub_toggle'  	=> 'ol',
				),
				'body_quote'    => array(
					'label'       	=> esc_html__( 'Blockquote', 'et_builder' ),
					'css'         	=> array(
						'main'  => "{$this->main_css_element}_popup .ddb-popup-body blockquote",
					),
					'text_shadow' 	=> array( 'show_if' => array( 'item_content' => 'custom' ) ),
					'toggle_slug' 	=> 'body',
					'sub_toggle'  	=> 'quote',
				),
			),
			'image_icon'     => array(
				'image_icon' => array(
					'label'           => et_builder_i18n( 'Image/Icon' ),
					'css'             => array(
						'main'    => '%%order_class%%_popup .ddb-popup-image-icon',
					),
					'tab_slug'        	=> 'advanced',
					'toggle_slug'     	=> 'popup_icon',
				),
				'popup' => array(
					'label'           => esc_html__( 'Popup', 'dd_builder' ),
					'css'             => array(
						'main'    => '%%order_class%%_popup .ddb-popup > div',
					),
					'margin_padding'  => array(
						'use_margin' => false,
					),
					'toggle_slug'     	=> 'popup',
				),
			),
			'text'           => array(
				'options'               => array(
					'text_orientation'  => array(
						'label'        	=> et_builder_i18n( 'Alignment' ),
						'options'      	=> et_builder_get_text_orientation_options( array( 'justified' ) ),
						'show_if' 		=> array('popup_type' => array('icon', 'button')),
						'toggle_slug' 	=> 'trigger',
					),
				),
			),
			'text_shadow'      => array(
				'default' => false,
			),
		);

		$this->custom_css_fields = array(
			'trigger_css' => array(
				'label' 	=> esc_html__( 'Trigger Element', 'dd_builder' ),
				'selector' 	=> '%%order_class%% .ddb-trigger-element',
				'show_if' 	=> array('popup_type' => array( 'icon', 'button') ),
			),
			'content_css' => array(
				'label'    	=> esc_html__( 'Popup Content', 'dd_builder' ),
				'selector' 	=> '%%order_class%%_popup .ddb-popup > div',
			),
			'title_css' => array(
				'label'    	=> et_builder_i18n( 'Title' ),
				'selector' 	=> '%%order_class%%_popup .ddb-popup-title',
				'show_if' 	=> array('item_content' => 'custom'),
			),
			'image_icon_css' => array(
				'label'    	=> et_builder_i18n( 'Image/Icon' ),
				'selector' 	=> '%%order_class%%_popup .ddb-popup-image-icon',
				'show_if' 	=> array( 'item_content' => 'custom', 'cc_icon_image' => array( 'icon', 'image') ),
			),
            'body_css' => array(
				'label'    	=> et_builder_i18n( 'Body' ),
				'selector' 	=> '%%order_class%%_popup .ddb-popup-body',
				'show_if' 	=> array('item_content' => 'custom'),
			),	
		);
	}

	public function get_fields() {
		$helper = new DonDiviBuilderFields();
		$fields = array(
			'popup_type' 	=> array(
				'label'           	=> esc_html__( 'Trigger On', 'dd_builder' ),
				'type'            	=> 'select',
				'options'			=> array(
					'load' 		=> esc_html__( 'Page Load', 'dd_builder' ),
					'scroll' 	=> esc_html__( 'Page Scroll', 'dd_builder' ),
					'icon'		=> et_builder_i18n( 'Image/Icon' ),
					'button'	=> esc_html__( 'Button', 'et_builder' ),
					'element' 	=> esc_html__( 'Element', 'et_builder' ),
					'exit' 		=> esc_html__( 'Exit Intent', 'dd_builder' ),
				),
				'default' 			=> 'load',
				'toggle_slug'     	=> 'main_content',				
			),
			'image_icon'    =>  array(
				'label'           	=> esc_html__( 'Image/Icon', 'et_builder' ),
				'type'            	=> 'multiple_buttons',
				'options'         	=>  array(
					'image' =>  array( 'title'  =>  esc_html__('Image', 'et_builder') ),
					'icon' 	=>  array( 'title' 	=>  esc_html__('Icon', 'et_builder') ),
				),
				'default'         	=> 'image',
				'show_if'		  	=> array( 'popup_type' => 'icon' ),
				'toggle_slug'     	=> 'main_content',
			),
			'font_icon' 		=> array(
				'label'           	=> esc_html__( 'Icon', 'et_builder' ),
				'type'            	=> 'select_icon',
				'class'           	=> array( 'et-pb-font-icon' ),
				'default'			=> '&#x4c;||divi||400',
				'show_if'		  	=> array( 'popup_type' => 'icon', 'image_icon' => 'icon' ),
				'mobile_options'  	=> true,
				'hover'				=> 'tabs',
				'toggle_slug'       => 'main_content',		
			),
			'image_src' 		=> array(
				'label'             => esc_html__( 'Image', 'et_builder' ),
				'type'              => 'upload',
				'upload_button_text'=> esc_attr__( 'Upload an image', 'et_builder' ),
				'choose_text'       => esc_attr__( 'Choose an Image', 'et_builder' ),
				'update_text'       => esc_attr__( 'Set As Image', 'et_builder' ),
				'default'			=> ET_BUILDER_PLACEHOLDER_LANDSCAPE_IMAGE_DATA,
				'show_if'		  	=> array( 'popup_type' => 'icon', 'image_icon' => 'image' ),
				'mobile_options'  	=> true,
				'hover'				=> 'tabs',
				'dynamic_content'   => 'image',
				'toggle_slug'       => 'main_content',
			),
			'button_text'              => array(
				'label'           	=> esc_html__( 'Button', 'et_builder' ),
				'description'     	=> esc_html__( 'Input your desired button text.', 'et_builder' ),
				'type'             	=> 'text',
				'default'          	=> 'Popup',
				'show_if' 			=> array('popup_type' => 'button'),
				'mobile_options'  	=> true,
				'hover'           	=> 'tabs',
				'dynamic_content' 	=> 'text',
				'toggle_slug'      	=> 'main_content',
			),
			'element_class' => array(
				'label'        	=> esc_html__( 'Element CSS Class', 'dd_builder' ),
				'description'  	=> esc_html__( 'Here you can enter the CSS class of the element that should trigger the popup.', 'dd_builder' ),
				'type'         	=> 'text',
				'show_if' 		=> array('popup_type' => 'element'),
				'toggle_slug'  	=> 'main_content',				
			),
			'page_scroll' => array(
				'label'      	=> esc_html__( 'Page Percentage', 'dd_builder' ),
				'type'         	=> 'range',
				'range_settings'   => array(
					'min'  => '10',
					'max'  => '100',
					'step' => '1',
				),
				'fixed_unit'   	=> '%',
				'default'      	=> '50%',
				'show_if' 		=> array('popup_type' => 'scroll'),
				'toggle_slug' 	=> 'main_content',
			),
			'delay' => array(
				'label'            => esc_html__( 'Popup Delay', 'dd_builder' ),
				'type'             => 'range',
				'option_category'  => 'layout',
				'range_settings'   => array(
					'min'  => '0',
					'max'  => '10000',
					'step' => '100',
				),
				'fixed_unit'       => 'ms',
				'default'          => '0ms',
				'toggle_slug'      => 'main_content',
			),
			'icon_color' => array(
				'label'           	=> esc_html__( 'Icon Color', 'et_builder' ),
				'description'     	=> esc_html__( 'Here you can define a custom color for your icon.', 'et_builder' ),
				'type'            	=> 'color-alpha',
				'default'         	=> et_builder_accent_color(),
				'show_if'			=> array( 'popup_type' => 'icon', 'image_icon' => 'icon'),
				'hover'           	=> 'tabs',
				'mobile_options'  	=> true,
				'sticky'          	=> true,
				'tab_slug'        	=> 'advanced',
				'toggle_slug'     	=> 'trigger',
			),
			'icon_width' 	=> array(
				'label'             => esc_html__( 'Image/Icon Width', 'et_builder' ),
				'description'     	=> esc_html__( 'Here you can choose icon/img width.', 'et_builder' ),
				'type'            	=> 'range',
				'default'         	=> '50px',
				'default_unit'    	=> 'px',
				'show_if'			=> array( 'popup_type' => 'icon', 'image_icon' => 'icon'),
				'mobile_options'  	=> true,
				'sticky'           	=> true,
				'tab_slug'         	=> 'advanced',
				'toggle_slug'      	=> 'trigger',
			),
			'image_width' 	 => array(
				'label'           	=> esc_html__( 'Image/Icon Width', 'et_builder' ),
				'description'      	=> esc_html__( 'Here you can choose icon/img width.', 'et_builder' ),
				'type'             	=> 'range',
				'allowed_values'  	=> et_builder_get_acceptable_css_string_values( 'width' ),
				'default'         	=> 'auto',
				'default_unit'    	=> '%',
				'show_if'			=> array( 'popup_type' => 'icon', 'image_icon' => 'image'),
				'mobile_options'   	=> true,
				'sticky'           	=> true,
				'tab_slug'         	=> 'advanced',
				'toggle_slug'      	=> 'trigger',
			),
			'popup_preview' => array( 
				'label'            	=> esc_html__( 'Preview in the Builder', 'dd_builder' ),
				'description'		=> esc_html__( 'Toggle this field to temporarily preview the popup.', 'dd_builder' ),
				'type'          	=> 'yes_no_button',
				'options'      		=> $helper->get_yes_no_button_options(),
				'default'			=> 'off',
				'toggle_slug'     	=> 'popup_content',
			),
			'item_content' 	=> array(
				'label'        		=> et_builder_i18n( 'Content' ),
				'type'         		=> 'select',
				'options'       	=> array(
					'custom' 	=> esc_html__( 'Custom', 'et_builder' ),
					'layout'  	=> esc_html__( 'Divi Layout', 'dd_builder' ),
				),
				'default'			=> 'custom',
				'toggle_slug'   	=> 'popup_content',
			),
			'divi_layout' => array(
				'label'           	=> esc_html__( 'Layout', 'et_builder' ),
				'description'     	=> esc_html__( 'Here you can choose the Divi Library saved layout.', 'dd_builder' ),
				'type'            	=> 'select',
				'options'			=> $helper->get_all_layouts(),
				'show_if' 			=> array('item_content' => 'layout'),
				'searchable'       	=> true,
				'displayRecent'    	=> false,
				'toggle_slug'     	=> 'popup_content',				
			),
		);
		$fields['image_alt'] = $helper->get_alt_field( array( 'popup_type' => 'icon', 'image_icon' => 'image'), 'Image Alt Text' );
		$popup_settings = array(
			'show_if' => array('item_content' => 'layout'),
			'show_if_custom' => array('item_content' => array('custom', 'layout')),
			'show_if_hover' => array('item_content' => array('custom', 'layout'), 'popup_type' => array( 'icon', 'button', 'element' )),
			'toggle_slug' => 'main_content',
			'advanced_slug' => 'popup'
		);
		$cc_settings = array(
			'content_slug' 	=> 'popup_content',
			'link_slug'		=> 'link_options',
			'icon'			=> true,
			'icon_slug'		=> 'popup_icon',
			'button'		=> true,
			'show_if_not' 	=> array('item_content' => 'layout'),
		);
		$fields = array_merge($fields,
			$helper->get_font_fields( 'body_ul', 'body_ol', 'body_quote', 'body'),
			$helper->get_popup_fields( $popup_settings ),	
			$helper->get_custom_content_fields( $cc_settings ),		
			$helper->get_effect_fields( 'effects', false )
		);
		return $fields;
	}

	public function multi_view_filter_value( $raw_value, $args ) {

		$name = isset( $args['name'] ) ? $args['name'] : '';

		if ( $raw_value && ( 'font_icon' === $name || 'popup_close_font_icon' === $name || 'cc_icon' === $name ) ) {
			return et_pb_get_extended_font_icon_value( $raw_value, true );
		}
		return $raw_value;
	}

	public function render( $attrs, $content, $render_slug ) {

		wp_enqueue_script( 'ddb-popup' );

		$this->add_classname( $this->get_text_orientation_classname() );

		$helper 			= DonDiviBuilderHelper::get();
		$multi_view 		= et_pb_multi_view_options( $this );
		$trigger_element 	= '';
		
		if ($this->props['popup_type'] === 'button') {
			$trigger_text = $this->_esc_attr( 'button_text', 'limited' );
			$trigger_element = $helper->get_custom_button( $this, 'trigger', 'button_text', $trigger_text, 'ddb-trigger-element' );
		} else if ( $this->props['popup_type'] === 'icon' && $this->props['image_icon'] === 'icon' )
			$trigger_element = $helper->render_icon( array( 
				'module' => $this, 
				'icon_attr' => 'font_icon', 
				'color_attr' => 'icon_color', 
				'size_attr' => 'icon_width', 
				'selector' =>  '%%order_class%% .ddb-trigger-element', 
				'attrs' => array( 'class' => 'ddb-trigger-element ddb-btn')
			) );
		else if ( $this->props['popup_type'] === 'icon' ) {
			$trigger_element = $helper->render_image( array(
				'module' 	=> $this, 
				'img_attr' 	=> 'image_src',
				'alt_attr'	=> 'image_alt',
				'attrs' => array( 'class' => 'ddb-trigger-element ddb-btn'),
			));
			$this->generate_styles(
				array(
					'base_attr_name' => 'image_width',
					'selector'       => '%%order_class%% .ddb-trigger-element',
					'css_property'   => 'width',
					'render_slug'    => $render_slug,
				)
			);
		}

		$popup_attrs = array( 
			'data-delay' => esc_attr( $this->props['delay'])
		);
		if ($this->props['popup_type'] === 'load') $popup_attrs['data-load'] = true;
		if ($this->props['popup_type'] === 'exit') $popup_attrs['data-exit-intent'] = true;
		if ($this->props['popup_type'] === 'icon' || $this->props['popup_type'] === 'button') {
			$popup_attrs['data-element-class'] = $this->get_module_order_class( $render_slug ) . ' .ddb-trigger-element';
		}
		if ($this->props['popup_type'] === 'element') {
			$popup_attrs['data-element-class'] = $this->props['element_class'];
		}
		if ($this->props['popup_type'] === 'scroll') {
			$popup_attrs['data-page-scroll'] = $this->props['page_scroll'];
		}

		$popup = $helper->render_popup( array(
			'module' 		=> $this,
			'slug'			=> 'popup',
			'layout_popup' 	=> $this->props['item_content'] === 'layout',
			'attrs' 		=> $popup_attrs,
		));

		return $popup . $trigger_element;
	}
	
}

new DDBUILDER_Popup_Module;
